"""
Python module generated from Java source file org.joml.Vector3fc

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Vector3fc:
    """
    Interface to a read-only view of a 3-dimensional vector of single-precision floats.

    Author(s)
    - Kai Burjack
    """

    def x(self) -> float:
        """
        Returns
        - the value of the x component
        """
        ...


    def y(self) -> float:
        """
        Returns
        - the value of the y component
        """
        ...


    def z(self) -> float:
        """
        Returns
        - the value of the z component
        """
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this vector into the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the vector is stored, use .get(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this vector in `x, y, z` order

        Returns
        - the passed in buffer

        See
        - .get(int, FloatBuffer)
        """
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this vector into the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: will receive the values of this vector in `x, y, z` order

        Returns
        - the passed in buffer
        """
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this vector into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is stored, use .get(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this vector in `x, y, z` order

        Returns
        - the passed in buffer

        See
        - .get(int, ByteBuffer)
        """
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this vector into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this vector in `x, y, z` order

        Returns
        - the passed in buffer
        """
        ...


    def getToAddress(self, address: int) -> "Vector3fc":
        """
        Store this vector at the given off-heap memory address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap address where to store this vector

        Returns
        - this
        """
        ...


    def sub(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Subtract the supplied vector from this one and store the result in `dest`.

        Arguments
        - v: the vector to subtract
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def sub(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Decrement the components of this vector by the given values and store the result in `dest`.

        Arguments
        - x: the x component to subtract
        - y: the y component to subtract
        - z: the z component to subtract
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Add the supplied vector to this one and store the result in `dest`.

        Arguments
        - v: the vector to add
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Increment the components of this vector by the given values and store the result in `dest`.

        Arguments
        - x: the x component to add
        - y: the y component to add
        - z: the z component to add
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def fma(self, a: "Vector3fc", b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Add the component-wise multiplication of `a * b` to this vector
        and store the result in `dest`.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def fma(self, a: float, b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Add the component-wise multiplication of `a * b` to this vector
        and store the result in `dest`.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulAdd(self, a: "Vector3fc", b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `dest`.

        Arguments
        - a: the multiplicand
        - b: the addend
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulAdd(self, a: float, b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `dest`.

        Arguments
        - a: the multiplicand
        - b: the addend
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply this vector component-wise by another vector and store the result in `dest`.

        Arguments
        - v: the vector to multiply by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def div(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Divide this vector component-wise by another vector and store the result in `dest`.

        Arguments
        - v: the vector to divide by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProject(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProject(self, mat: "Matrix4fc", w: float, dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method uses the given `w` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - w: the w component to use
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method assumes that the matrix `mat` represents only an affine transformation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method assumes that the matrix `mat` represents only an affine transformation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix3fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix with this vector and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix3dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix with this vector and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix3x2fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with `this` by assuming a
        third row in the matrix of `(0, 0, 1)` and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix3x2dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given matrix `mat` with `this` by assuming a
        third row in the matrix of `(0, 0, 1)` and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulTranspose(self, mat: "Matrix3fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the transpose of the given matrix with this vector and store the result in `dest`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPosition(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPosition(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPosition(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPosition(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionGeneric(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionGeneric(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulTransposePosition(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulTransposePosition(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulPositionW(self, mat: "Matrix4fc", dest: "Vector3f") -> float:
        """
        Multiply the given 4x4 matrix `mat` with `this`, store the
        result in `dest` and return the *w* component of the resulting 4D vector.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the `(x, y, z)` components of the resulting vector

        Returns
        - the *w* component of the resulting 4D vector after multiplication
        """
        ...


    def mulPositionW(self, mat: "Matrix4dc", dest: "Vector3f") -> float:
        """
        Multiply the given 4x4 matrix `mat` with `this`, store the
        result in `dest` and return the *w* component of the resulting 4D vector.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the `(x, y, z)` components of the resulting vector

        Returns
        - the *w* component of the resulting 4D vector after multiplication
        """
        ...


    def mulDirection(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulDirection(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulDirection(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulDirection(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulTransposeDirection(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this` and store the
        result in `dest`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, scalar: float, dest: "Vector3f") -> "Vector3f":
        """
        Multiply all components of this vector by the given scalar
        value and store the result in `dest`.

        Arguments
        - scalar: the scalar to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Multiply the components of this vector by the given scalar values and store the result in `dest`.

        Arguments
        - x: the x component to multiply this vector by
        - y: the y component to multiply this vector by
        - z: the z component to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def div(self, scalar: float, dest: "Vector3f") -> "Vector3f":
        """
        Divide all components of this vector by the given scalar
        value and store the result in `dest`.

        Arguments
        - scalar: the scalar to divide by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def div(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Divide the components of this vector by the given scalar values and store the result in `dest`.

        Arguments
        - x: the x component to divide this vector by
        - y: the y component to divide this vector by
        - z: the z component to divide this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotate(self, quat: "Quaternionfc", dest: "Vector3f") -> "Vector3f":
        """
        Rotate this vector by the given quaternion `quat` and store the result in `dest`.

        Arguments
        - quat: the quaternion to rotate this vector
        - dest: will hold the result

        Returns
        - dest

        See
        - Quaternionfc.transform(Vector3f)
        """
        ...


    def rotationTo(self, toDir: "Vector3fc", dest: "Quaternionf") -> "Quaternionf":
        """
        Compute the quaternion representing a rotation of `this` vector to point along `toDir`
        and store the result in `dest`.
        
        Because there can be multiple possible rotations, this method chooses the one with the shortest arc.

        Arguments
        - toDir: the destination direction
        - dest: will hold the result

        Returns
        - dest

        See
        - Quaternionf.rotationTo(Vector3fc, Vector3fc)
        """
        ...


    def rotationTo(self, toDirX: float, toDirY: float, toDirZ: float, dest: "Quaternionf") -> "Quaternionf":
        """
        Compute the quaternion representing a rotation of `this` vector to point along `(toDirX, toDirY, toDirZ)`
        and store the result in `dest`.
        
        Because there can be multiple possible rotations, this method chooses the one with the shortest arc.

        Arguments
        - toDirX: the x coordinate of the destination direction
        - toDirY: the y coordinate of the destination direction
        - toDirZ: the z coordinate of the destination direction
        - dest: will hold the result

        Returns
        - dest

        See
        - Quaternionf.rotationTo(float, float, float, float, float, float)
        """
        ...


    def rotateAxis(self, angle: float, aX: float, aY: float, aZ: float, dest: "Vector3f") -> "Vector3f":
        """
        Rotate this vector the specified radians around the given rotation axis and store the result
        into `dest`.
        
        If the rotation axis is either `(1, 0, 0)`, `(0, 1, 0)` or `(0, 0, 1)`,
        then .rotateX(float, Vector3f) rotateX(), .rotateY(float, Vector3f) rotateY() or
        .rotateZ(float, Vector3f) rotateZ(), respectively, should be used instead.

        Arguments
        - angle: the angle in radians
        - aX: the x component of the rotation axis
        - aY: the y component of the rotation axis
        - aZ: the z component of the rotation axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateX(self, angle: float, dest: "Vector3f") -> "Vector3f":
        """
        Rotate this vector the specified radians around the X axis and store the result
        into `dest`.

        Arguments
        - angle: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateY(self, angle: float, dest: "Vector3f") -> "Vector3f":
        """
        Rotate this vector the specified radians around the Y axis and store the result
        into `dest`.

        Arguments
        - angle: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateZ(self, angle: float, dest: "Vector3f") -> "Vector3f":
        """
        Rotate this vector the specified radians around the Z axis and store the result
        into `dest`.

        Arguments
        - angle: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def lengthSquared(self) -> float:
        """
        Return the length squared of this vector.

        Returns
        - the length squared
        """
        ...


    def length(self) -> float:
        """
        Return the length of this vector.

        Returns
        - the length
        """
        ...


    def normalize(self, dest: "Vector3f") -> "Vector3f":
        """
        Normalize this vector and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def normalize(self, length: float, dest: "Vector3f") -> "Vector3f":
        """
        Scale this vector to have the given length and store the result in `dest`.

        Arguments
        - length: the desired length
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def cross(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Compute the cross product of this vector and `v` and store the result in `dest`.

        Arguments
        - v: the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def cross(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Compute the cross product of this vector and `(x, y, z)` and store the result in `dest`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def distance(self, v: "Vector3fc") -> float:
        """
        Return the distance between this Vector and `v`.

        Arguments
        - v: the other vector

        Returns
        - the distance
        """
        ...


    def distance(self, x: float, y: float, z: float) -> float:
        """
        Return the distance between `this` vector and `(x, y, z)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector

        Returns
        - the euclidean distance
        """
        ...


    def distanceSquared(self, v: "Vector3fc") -> float:
        """
        Return the square of the distance between this vector and `v`.

        Arguments
        - v: the other vector

        Returns
        - the squared of the distance
        """
        ...


    def distanceSquared(self, x: float, y: float, z: float) -> float:
        """
        Return the square of the distance between `this` vector and `(x, y, z)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector

        Returns
        - the square of the distance
        """
        ...


    def dot(self, v: "Vector3fc") -> float:
        """
        Return the dot product of this vector and the supplied vector.

        Arguments
        - v: the other vector

        Returns
        - the dot product
        """
        ...


    def dot(self, x: float, y: float, z: float) -> float:
        """
        Return the dot product of this vector and the vector `(x, y, z)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector

        Returns
        - the dot product
        """
        ...


    def angleCos(self, v: "Vector3fc") -> float:
        """
        Return the cosine of the angle between this vector and the supplied vector. Use this instead of Math.cos(this.angle(v)).

        Arguments
        - v: the other vector

        Returns
        - the cosine of the angle

        See
        - .angle(Vector3fc)
        """
        ...


    def angle(self, v: "Vector3fc") -> float:
        """
        Return the angle between this vector and the supplied vector.

        Arguments
        - v: the other vector

        Returns
        - the angle, in radians

        See
        - .angleCos(Vector3fc)
        """
        ...


    def angleSigned(self, v: "Vector3fc", n: "Vector3fc") -> float:
        """
        Return the signed angle between this vector and the supplied vector with
        respect to the plane with the given normal vector `n`.

        Arguments
        - v: the other vector
        - n: the plane's normal vector

        Returns
        - the angle, in radians

        See
        - .angleCos(Vector3fc)
        """
        ...


    def angleSigned(self, x: float, y: float, z: float, nx: float, ny: float, nz: float) -> float:
        """
        Return the signed angle between this vector and the supplied vector with
        respect to the plane with the given normal vector `(nx, ny, nz)`.

        Arguments
        - x: the x coordinate of the other vector
        - y: the y coordinate of the other vector
        - z: the z coordinate of the other vector
        - nx: the x coordinate of the plane's normal vector
        - ny: the y coordinate of the plane's normal vector
        - nz: the z coordinate of the plane's normal vector

        Returns
        - the angle, in radians
        """
        ...


    def min(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Set the components of `dest` to be the component-wise minimum of this and the other vector.

        Arguments
        - v: the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def max(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Set the components of `dest` to be the component-wise maximum of this and the other vector.

        Arguments
        - v: the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def negate(self, dest: "Vector3f") -> "Vector3f":
        """
        Negate this vector and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def absolute(self, dest: "Vector3f") -> "Vector3f":
        """
        Compute the absolute values of the individual components of `this` and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def reflect(self, normal: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Reflect this vector about the given `normal` vector and store the result in `dest`.

        Arguments
        - normal: the vector to reflect about
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def reflect(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Reflect this vector about the given normal vector and store the result in `dest`.

        Arguments
        - x: the x component of the normal
        - y: the y component of the normal
        - z: the z component of the normal
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def half(self, other: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Compute the half vector between this and the other vector and store the result in `dest`.

        Arguments
        - other: the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def half(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        """
        Compute the half vector between this and the vector `(x, y, z)` 
        and store the result in `dest`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def smoothStep(self, v: "Vector3fc", t: float, dest: "Vector3f") -> "Vector3f":
        """
        Compute a smooth-step (i.e. hermite with zero tangents) interpolation
        between `this` vector and the given vector `v` and
        store the result in `dest`.

        Arguments
        - v: the other vector
        - t: the interpolation factor, within `[0..1]`
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def hermite(self, t0: "Vector3fc", v1: "Vector3fc", t1: "Vector3fc", t: float, dest: "Vector3f") -> "Vector3f":
        """
        Compute a hermite interpolation between `this` vector with its
        associated tangent `t0` and the given vector `v`
        with its tangent `t1` and store the result in
        `dest`.

        Arguments
        - t0: the tangent of `this` vector
        - v1: the other vector
        - t1: the tangent of the other vector
        - t: the interpolation factor, within `[0..1]`
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def lerp(self, other: "Vector3fc", t: float, dest: "Vector3f") -> "Vector3f":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `dest`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other vector
        - t: the interpolation factor between 0.0 and 1.0
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, component: int) -> float:
        """
        Get the value of the specified component of this vector.

        Arguments
        - component: the component, within `[0..2]`

        Returns
        - the value

        Raises
        - IllegalArgumentException: if `component` is not within `[0..2]`
        """
        ...


    def get(self, mode: int, dest: "Vector3i") -> "Vector3i":
        """
        Set the components of the given vector `dest` to those of `this` vector
        using the given RoundingMode.

        Arguments
        - mode: the RoundingMode to use
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, dest: "Vector3f") -> "Vector3f":
        """
        Set the components of the given vector `dest` to those of `this` vector.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, dest: "Vector3d") -> "Vector3d":
        """
        Set the components of the given vector `dest` to those of `this` vector.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def maxComponent(self) -> int:
        """
        Determine the component with the biggest absolute value.

        Returns
        - the component index, within `[0..2]`
        """
        ...


    def minComponent(self) -> int:
        """
        Determine the component with the smallest (towards zero) absolute value.

        Returns
        - the component index, within `[0..2]`
        """
        ...


    def orthogonalize(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Transform `this` vector so that it is orthogonal to the given vector `v`, normalize the result and store it into `dest`.
        
        Reference: <a href="https://en.wikipedia.org/wiki/Gram%E2%80%93Schmidt_process">Gram–Schmidt process</a>

        Arguments
        - v: the reference vector which the result should be orthogonal to
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def orthogonalizeUnit(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        """
        Transform `this` vector so that it is orthogonal to the given unit vector `v`, normalize the result and store it into `dest`.
        
        The vector `v` is assumed to be a .normalize(Vector3f) unit vector.
        
        Reference: <a href="https://en.wikipedia.org/wiki/Gram%E2%80%93Schmidt_process">Gram–Schmidt process</a>

        Arguments
        - v: the reference unit vector which the result should be orthogonal to
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def floor(self, dest: "Vector3f") -> "Vector3f":
        """
        Compute for each component of this vector the largest (closest to positive
        infinity) `float` value that is less than or equal to that
        component and is equal to a mathematical integer and store the result in
        `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def ceil(self, dest: "Vector3f") -> "Vector3f":
        """
        Compute for each component of this vector the smallest (closest to negative
        infinity) `float` value that is greater than or equal to that
        component and is equal to a mathematical integer and store the result in
        `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def round(self, dest: "Vector3f") -> "Vector3f":
        """
        Compute for each component of this vector the closest float that is equal to
        a mathematical integer, with ties rounding to positive infinity and store
        the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def isFinite(self) -> bool:
        """
        Determine whether all components are finite floating-point values, that
        is, they are not Float.isNaN() NaN and not
        Float.isInfinite() infinity.

        Returns
        - `True` if all components are finite floating-point values;
                `False` otherwise
        """
        ...


    def equals(self, v: "Vector3fc", delta: float) -> bool:
        """
        Compare the vector components of `this` vector with the given vector using the given `delta`
        and return whether all of them are equal within a maximum difference of `delta`.
        
        Please note that this method is not used by any data structure such as ArrayList HashSet or HashMap
        and their operations, such as ArrayList.contains(Object) or HashSet.remove(Object), since those
        data structures only use the Object.equals(Object) and Object.hashCode() methods.

        Arguments
        - v: the other vector
        - delta: the allowed maximum difference

        Returns
        - `True` whether all of the vector components are equal; `False` otherwise
        """
        ...


    def equals(self, x: float, y: float, z: float) -> bool:
        """
        Compare the vector components of `this` vector with the given `(x, y, z)`
        and return whether all of them are equal.

        Arguments
        - x: the x component to compare to
        - y: the y component to compare to
        - z: the z component to compare to

        Returns
        - `True` if all the vector components are equal
        """
        ...



"""
Python module generated from Java source file org.bukkit.configuration.file.YamlConfigurationOptions

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.configuration.file import *
from typing import Any, Callable, Iterable, Tuple


class YamlConfigurationOptions(FileConfigurationOptions):
    """
    Various settings for controlling the input and output of a YamlConfiguration
    """

    def configuration(self) -> "YamlConfiguration":
        ...


    def copyDefaults(self, value: bool) -> "YamlConfigurationOptions":
        ...


    def pathSeparator(self, value: str) -> "YamlConfigurationOptions":
        ...


    def setHeader(self, value: list[str]) -> "YamlConfigurationOptions":
        ...


    def header(self, value: str) -> "YamlConfigurationOptions":
        ...


    def setFooter(self, value: list[str]) -> "YamlConfigurationOptions":
        ...


    def parseComments(self, value: bool) -> "YamlConfigurationOptions":
        ...


    def copyHeader(self, value: bool) -> "YamlConfigurationOptions":
        ...


    def indent(self) -> int:
        """
        Gets how much spaces should be used to indent each line.
        
        The minimum value this may be is 2, and the maximum is 9.

        Returns
        - How much to indent by
        """
        ...


    def indent(self, value: int) -> "YamlConfigurationOptions":
        """
        Sets how much spaces should be used to indent each line.
        
        The minimum value this may be is 2, and the maximum is 9.

        Arguments
        - value: New indent

        Returns
        - This object, for chaining
        """
        ...


    def width(self) -> int:
        """
        Gets how long a line can be, before it gets split.

        Returns
        - How the max line width
        """
        ...


    def width(self, value: int) -> "YamlConfigurationOptions":
        """
        Sets how long a line can be, before it gets split.

        Arguments
        - value: New width

        Returns
        - This object, for chaining
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.EndGateway

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.block import *
from typing import Any, Callable, Iterable, Tuple


class EndGateway(TileState):
    """
    Represents a captured state of an end gateway.
    """

    def getExitLocation(self) -> "Location":
        """
        Gets the location that entities are teleported to when
        entering the gateway portal.
        
        If this block state is not placed the location's world will be null.

        Returns
        - the gateway exit location
        """
        ...


    def setExitLocation(self, location: "Location") -> None:
        """
        Sets the exit location that entities are teleported to when
        they enter the gateway portal.
        
        If this block state is not placed the location's world has to be null.

        Arguments
        - location: the new exit location

        Raises
        - IllegalArgumentException: for differing worlds
        """
        ...


    def isExactTeleport(self) -> bool:
        """
        Gets whether this gateway will teleport entities directly to
        the exit location instead of finding a nearby location.

        Returns
        - True if the gateway is teleporting to the exact location
        """
        ...


    def setExactTeleport(self, exact: bool) -> None:
        """
        Sets whether this gateway will teleport entities directly to
        the exit location instead of finding a nearby location.

        Arguments
        - exact: whether to teleport to the exact location
        """
        ...


    def getAge(self) -> int:
        """
        Gets the age in ticks of the gateway.
        
        If the age is less than 200 ticks a magenta beam will be emitted, whilst
        if it is a multiple of 2400 ticks a purple beam will be emitted.

        Returns
        - age in ticks
        """
        ...


    def setAge(self, age: int) -> None:
        """
        Sets the age in ticks of the gateway.
        
        If the age is less than 200 ticks a magenta beam will be emitted, whilst
        if it is a multiple of 2400 ticks a purple beam will be emitted.

        Arguments
        - age: new age in ticks
        """
        ...



"""
Python module generated from Java source file org.bukkit.Bukkit

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableList
from java.io import File
from java.io import Serializable
from java.net import InetAddress
from java.util import Collections
from java.util import Iterator
from java.util import UUID
from java.util.function import Consumer
from org.bukkit import *
from org.bukkit.Warning import WarningState
from org.bukkit.advancement import Advancement
from org.bukkit.block.data import BlockData
from org.bukkit.boss import BarColor
from org.bukkit.boss import BarFlag
from org.bukkit.boss import BarStyle
from org.bukkit.boss import BossBar
from org.bukkit.boss import KeyedBossBar
from org.bukkit.command import CommandException
from org.bukkit.command import CommandSender
from org.bukkit.command import ConsoleCommandSender
from org.bukkit.command import PluginCommand
from org.bukkit.entity import Entity
from org.bukkit.entity import EntityFactory
from org.bukkit.entity import EntitySnapshot
from org.bukkit.entity import Player
from org.bukkit.entity import SpawnCategory
from org.bukkit.event.inventory import InventoryType
from org.bukkit.event.server import ServerListPingEvent
from org.bukkit.generator import ChunkGenerator
from org.bukkit.help import HelpMap
from org.bukkit.inventory import Inventory
from org.bukkit.inventory import InventoryHolder
from org.bukkit.inventory import ItemCraftResult
from org.bukkit.inventory import ItemFactory
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory import MenuType
from org.bukkit.inventory import Merchant
from org.bukkit.inventory import Recipe
from org.bukkit.inventory.meta import ItemMeta
from org.bukkit.loot import LootTable
from org.bukkit.map import MapView
from org.bukkit.packs import DataPackManager
from org.bukkit.packs import ResourcePack
from org.bukkit.permissions import Permissible
from org.bukkit.plugin import PluginManager
from org.bukkit.plugin import ServicesManager
from org.bukkit.plugin.messaging import Messenger
from org.bukkit.profile import PlayerProfile
from org.bukkit.scheduler import BukkitScheduler
from org.bukkit.scoreboard import Criteria
from org.bukkit.scoreboard import ScoreboardManager
from org.bukkit.structure import StructureManager
from org.bukkit.util import CachedServerIcon
from typing import Any, Callable, Iterable, Tuple


class Bukkit:
    """
    Represents the Bukkit core, for version and Server singleton handling
    """

    @staticmethod
    def getServer() -> "Server":
        """
        Gets the current Server singleton

        Returns
        - Server instance being ran
        """
        ...


    @staticmethod
    def setServer(server: "Server") -> None:
        """
        Attempts to set the Server singleton.
        
        This cannot be done if the Server is already set.

        Arguments
        - server: Server instance
        """
        ...


    @staticmethod
    def getName() -> str:
        """
        Gets the name of this server implementation.

        Returns
        - name of this server implementation
        """
        ...


    @staticmethod
    def getVersion() -> str:
        """
        Gets the version string of this server implementation.

        Returns
        - version of this server implementation
        """
        ...


    @staticmethod
    def getBukkitVersion() -> str:
        """
        Gets the Bukkit version that this server is running.

        Returns
        - version of Bukkit
        """
        ...


    @staticmethod
    def getOnlinePlayers() -> Iterable["Player"]:
        """
        Gets a view of all currently logged in players. This Collections.unmodifiableCollection(Collection) view is a reused
        object, making some operations like Collection.size()
        zero-allocation.
        
        The collection is a view backed by the internal representation, such
        that, changes to the internal state of the server will be reflected
        immediately. However, the reuse of the returned collection (identity)
        is not strictly guaranteed for future or all implementations. Casting
        the collection, or relying on interface implementations (like Serializable or List), is deprecated.
        
        Iteration behavior is undefined outside of self-contained main-thread
        uses. Normal and immediate iterator use without consequences that
        affect the collection are fully supported. The effects following
        (non-exhaustive) Entity.teleport(Location) teleportation,
        Player.setHealth(double) death, and Player.kickPlayer(
        String) kicking are undefined. Any use of this collection from
        asynchronous threads is unsafe.
        
        For safe consequential iteration or mimicking the old array behavior,
        using Collection.toArray(Object[]) is recommended. For making
        snapshots, ImmutableList.copyOf(Collection) is recommended.

        Returns
        - a view of currently online players.
        """
        ...


    @staticmethod
    def getMaxPlayers() -> int:
        """
        Get the maximum amount of players which can login to this server.

        Returns
        - the amount of players this server allows
        """
        ...


    @staticmethod
    def setMaxPlayers(maxPlayers: int) -> None:
        """
        Set the maximum amount of players allowed to be logged in at once.

        Arguments
        - maxPlayers: The maximum amount of concurrent players
        """
        ...


    @staticmethod
    def getPort() -> int:
        """
        Get the game port that the server runs on.

        Returns
        - the port number of this server
        """
        ...


    @staticmethod
    def getViewDistance() -> int:
        """
        Get the view distance from this server.

        Returns
        - the view distance from this server.
        """
        ...


    @staticmethod
    def getSimulationDistance() -> int:
        """
        Get the simulation distance from this server.

        Returns
        - the simulation distance from this server.
        """
        ...


    @staticmethod
    def getIp() -> str:
        """
        Get the IP that this server is bound to, or empty string if not
        specified.

        Returns
        - the IP string that this server is bound to, otherwise empty
            string
        """
        ...


    @staticmethod
    def getWorldType() -> str:
        """
        Get world type (level-type setting) for default world.

        Returns
        - the value of level-type (e.g. DEFAULT, FLAT, DEFAULT_1_1)
        """
        ...


    @staticmethod
    def getGenerateStructures() -> bool:
        """
        Get generate-structures setting.

        Returns
        - True if structure generation is enabled, False otherwise
        """
        ...


    @staticmethod
    def getMaxWorldSize() -> int:
        """
        Get max world size.

        Returns
        - the maximum world size as specified for the server
        """
        ...


    @staticmethod
    def getAllowEnd() -> bool:
        """
        Gets whether this server allows the End or not.

        Returns
        - whether this server allows the End or not
        """
        ...


    @staticmethod
    def getAllowNether() -> bool:
        """
        Gets whether this server allows the Nether or not.

        Returns
        - whether this server allows the Nether or not
        """
        ...


    @staticmethod
    def isLoggingIPs() -> bool:
        """
        Gets whether the server is logging the IP addresses of players.

        Returns
        - whether the server is logging the IP addresses of players
        """
        ...


    @staticmethod
    def getInitialEnabledPacks() -> list[str]:
        ...


    @staticmethod
    def getInitialDisabledPacks() -> list[str]:
        ...


    @staticmethod
    def getDataPackManager() -> "DataPackManager":
        """
        Get the DataPack Manager.

        Returns
        - the manager
        """
        ...


    @staticmethod
    def getServerResourcePack() -> "ResourcePack":
        """
        Gets the resource pack configured to be sent to clients by the server.

        Returns
        - the resource pack
        """
        ...


    @staticmethod
    def getServerTickManager() -> "ServerTickManager":
        """
        Get the ServerTick Manager.

        Returns
        - the manager
        """
        ...


    @staticmethod
    def getResourcePack() -> str:
        """
        Gets the server resource pack uri, or empty string if not specified.

        Returns
        - the server resource pack uri, otherwise empty string
        """
        ...


    @staticmethod
    def getResourcePackHash() -> str:
        """
        Gets the SHA-1 digest of the server resource pack, or empty string if
        not specified.

        Returns
        - the SHA-1 digest of the server resource pack, otherwise empty
            string
        """
        ...


    @staticmethod
    def getResourcePackPrompt() -> str:
        """
        Gets the custom prompt message to be shown when the server resource
        pack is required, or empty string if not specified.

        Returns
        - the custom prompt message to be shown when the server resource,
            otherwise empty string
        """
        ...


    @staticmethod
    def isResourcePackRequired() -> bool:
        """
        Gets whether the server resource pack is enforced.

        Returns
        - whether the server resource pack is enforced
        """
        ...


    @staticmethod
    def hasWhitelist() -> bool:
        """
        Gets whether this server has a whitelist or not.

        Returns
        - whether this server has a whitelist or not
        """
        ...


    @staticmethod
    def setWhitelist(value: bool) -> None:
        """
        Sets if the server is whitelisted.

        Arguments
        - value: True for whitelist on, False for off
        """
        ...


    @staticmethod
    def isWhitelistEnforced() -> bool:
        """
        Gets whether the server whitelist is enforced.
        
        If the whitelist is enforced, non-whitelisted players will be
        disconnected when the server whitelist is reloaded.

        Returns
        - whether the server whitelist is enforced
        """
        ...


    @staticmethod
    def setWhitelistEnforced(value: bool) -> None:
        """
        Sets if the server whitelist is enforced.
        
        If the whitelist is enforced, non-whitelisted players will be
        disconnected when the server whitelist is reloaded.

        Arguments
        - value: True for enforced, False for not
        """
        ...


    @staticmethod
    def getWhitelistedPlayers() -> set["OfflinePlayer"]:
        """
        Gets a list of whitelisted players.

        Returns
        - a set containing all whitelisted players
        """
        ...


    @staticmethod
    def reloadWhitelist() -> None:
        """
        Reloads the whitelist from disk.
        """
        ...


    @staticmethod
    def broadcastMessage(message: str) -> int:
        """
        Broadcast a message to all players.
        
        This is the same as calling .broadcast(java.lang.String,
        java.lang.String) to Server.BROADCAST_CHANNEL_USERS

        Arguments
        - message: the message

        Returns
        - the number of players
        """
        ...


    @staticmethod
    def getUpdateFolder() -> str:
        """
        Gets the name of the update folder. The update folder is used to safely
        update plugins at the right moment on a plugin load.
        
        The update folder name is relative to the plugins folder.

        Returns
        - the name of the update folder
        """
        ...


    @staticmethod
    def getUpdateFolderFile() -> "File":
        """
        Gets the update folder. The update folder is used to safely update
        plugins at the right moment on a plugin load.

        Returns
        - the update folder
        """
        ...


    @staticmethod
    def getConnectionThrottle() -> int:
        """
        Gets the value of the connection throttle setting.

        Returns
        - the value of the connection throttle setting
        """
        ...


    @staticmethod
    def getTicksPerAnimalSpawns() -> int:
        """
        Gets default ticks per animal spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn monsters
            every tick.
        - A value of 400 will mean the server will attempt to spawn monsters
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, animal spawning will be disabled. We
        recommend using spawn-animals to control this instead.
        
        Minecraft default: 400.

        Returns
        - the default ticks per animal spawns value

        Deprecated
        - Deprecated in favor of .getTicksPerSpawns(SpawnCategory)
        """
        ...


    @staticmethod
    def getTicksPerMonsterSpawns() -> int:
        """
        Gets the default ticks per monster spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn monsters
            every tick.
        - A value of 400 will mean the server will attempt to spawn monsters
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, monsters spawning will be disabled. We
        recommend using spawn-monsters to control this instead.
        
        Minecraft default: 1.

        Returns
        - the default ticks per monsters spawn value

        Deprecated
        - Deprecated in favor of .getTicksPerSpawns(SpawnCategory)
        """
        ...


    @staticmethod
    def getTicksPerWaterSpawns() -> int:
        """
        Gets the default ticks per water mob spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn water mobs
            every tick.
        - A value of 400 will mean the server will attempt to spawn water mobs
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, water mobs spawning will be disabled.
        
        Minecraft default: 1.

        Returns
        - the default ticks per water mobs spawn value

        Deprecated
        - Deprecated in favor of .getTicksPerSpawns(SpawnCategory)
        """
        ...


    @staticmethod
    def getTicksPerAmbientSpawns() -> int:
        """
        Gets the default ticks per ambient mob spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn ambient mobs
            every tick.
        - A value of 400 will mean the server will attempt to spawn ambient mobs
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, ambient mobs spawning will be disabled.
        
        Minecraft default: 1.

        Returns
        - the default ticks per ambient mobs spawn value

        Deprecated
        - Deprecated in favor of .getTicksPerSpawns(SpawnCategory)
        """
        ...


    @staticmethod
    def getTicksPerWaterAmbientSpawns() -> int:
        """
        Gets the default ticks per water ambient mob spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn water ambient mobs
            every tick.
        - A value of 400 will mean the server will attempt to spawn water ambient mobs
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, ambient mobs spawning will be disabled.
        
        Minecraft default: 1.

        Returns
        - the default ticks per water ambient mobs spawn value

        Deprecated
        - Deprecated in favor of .getTicksPerSpawns(SpawnCategory)
        """
        ...


    @staticmethod
    def getTicksPerWaterUndergroundCreatureSpawns() -> int:
        """
        Gets the default ticks per water underground creature spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn water underground creature
            every tick.
        - A value of 400 will mean the server will attempt to spawn water underground creature
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, water underground creature spawning will be disabled.
        
        Minecraft default: 1.

        Returns
        - the default ticks per water underground creature spawn value

        Deprecated
        - Deprecated in favor of .getTicksPerSpawns(SpawnCategory)
        """
        ...


    @staticmethod
    def getTicksPerSpawns(spawnCategory: "SpawnCategory") -> int:
        """
        Gets the default ticks per SpawnCategory spawns value.
        
        **Example Usage:**
        
        - A value of 1 will mean the server will attempt to spawn SpawnCategory mobs
            every tick.
        - A value of 400 will mean the server will attempt to spawn SpawnCategory mobs
            every 400th tick.
        - A value below 0 will be reset back to Minecraft's default.
        
        
        **Note:** If set to 0, SpawnCategory mobs spawning will be disabled.
        
        Minecraft default: 1.
        
        **Note: ** the SpawnCategory.MISC are not consider.

        Arguments
        - spawnCategory: the category of spawn

        Returns
        - the default ticks per SpawnCategory mobs spawn value
        """
        ...


    @staticmethod
    def getPlayer(name: str) -> "Player":
        """
        Gets a player whose name matches the given name closest.
        
        Use .getPlayerExact(String) to get the player matching the input exactly
        and .matchPlayer(String) if you want a list of all players matching the input.
        
        This method may not return objects for offline players.

        Arguments
        - name: the name to look up

        Returns
        - a player if one was found, null otherwise
        """
        ...


    @staticmethod
    def getPlayerExact(name: str) -> "Player":
        """
        Gets the player with the exact given name, case insensitive.

        Arguments
        - name: Exact name of the player to retrieve

        Returns
        - a player object if one was found, null otherwise
        """
        ...


    @staticmethod
    def matchPlayer(name: str) -> list["Player"]:
        """
        Attempts to match any players with the given name, and returns a list
        of all possibly matches.
        
        This list is not sorted in any particular order. If an exact match is
        found, the returned list will only contain a single result.

        Arguments
        - name: the (partial) name to match

        Returns
        - list of all possible players
        """
        ...


    @staticmethod
    def getPlayer(id: "UUID") -> "Player":
        """
        Gets the player with the given UUID.

        Arguments
        - id: UUID of the player to retrieve

        Returns
        - a player object if one was found, null otherwise
        """
        ...


    @staticmethod
    def getPluginManager() -> "PluginManager":
        """
        Gets the plugin manager for interfacing with plugins.

        Returns
        - a plugin manager for this Server instance
        """
        ...


    @staticmethod
    def getScheduler() -> "BukkitScheduler":
        """
        Gets the scheduler for managing scheduled events.

        Returns
        - a scheduling service for this server
        """
        ...


    @staticmethod
    def getServicesManager() -> "ServicesManager":
        """
        Gets a services manager.

        Returns
        - s services manager
        """
        ...


    @staticmethod
    def getWorlds() -> list["World"]:
        """
        Gets a list of all worlds on this server.

        Returns
        - a list of worlds
        """
        ...


    @staticmethod
    def createWorld(creator: "WorldCreator") -> "World":
        """
        Creates or loads a world with the given name using the specified
        options.
        
        If the world is already loaded, it will just return the equivalent of
        getWorld(creator.name()).

        Arguments
        - creator: the options to use when creating the world

        Returns
        - newly created or loaded world
        """
        ...


    @staticmethod
    def unloadWorld(name: str, save: bool) -> bool:
        """
        Unloads a world with the given name.

        Arguments
        - name: Name of the world to unload
        - save: whether to save the chunks before unloading

        Returns
        - True if successful, False otherwise
        """
        ...


    @staticmethod
    def unloadWorld(world: "World", save: bool) -> bool:
        """
        Unloads the given world.

        Arguments
        - world: the world to unload
        - save: whether to save the chunks before unloading

        Returns
        - True if successful, False otherwise
        """
        ...


    @staticmethod
    def getWorld(name: str) -> "World":
        """
        Gets the world with the given name.

        Arguments
        - name: the name of the world to retrieve

        Returns
        - a world with the given name, or null if none exists
        """
        ...


    @staticmethod
    def getWorld(uid: "UUID") -> "World":
        """
        Gets the world from the given Unique ID.

        Arguments
        - uid: a unique-id of the world to retrieve

        Returns
        - a world with the given Unique ID, or null if none exists
        """
        ...


    @staticmethod
    def createWorldBorder() -> "WorldBorder":
        """
        Create a new virtual WorldBorder.

        Returns
        - the created world border instance

        See
        - Player.setWorldBorder(WorldBorder)
        """
        ...


    @staticmethod
    def getMap(id: int) -> "MapView":
        """
        Gets the map from the given item ID.

        Arguments
        - id: the id of the map to get

        Returns
        - a map view if it exists, or null otherwise

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def createMap(world: "World") -> "MapView":
        """
        Create a new map with an automatically assigned ID.

        Arguments
        - world: the world the map will belong to

        Returns
        - a newly created map view
        """
        ...


    @staticmethod
    def createExplorerMap(world: "World", location: "Location", structureType: "StructureType") -> "ItemStack":
        """
        Create a new explorer map targeting the closest nearby structure of a
        given StructureType.
        
        This method uses implementation default values for radius and
        findUnexplored (usually 100, True).

        Arguments
        - world: the world the map will belong to
        - location: the origin location to find the nearest structure
        - structureType: the type of structure to find

        Returns
        - a newly created item stack

        See
        - World.locateNearestStructure(org.bukkit.Location,
             org.bukkit.StructureType, int, boolean)
        """
        ...


    @staticmethod
    def createExplorerMap(world: "World", location: "Location", structureType: "StructureType", radius: int, findUnexplored: bool) -> "ItemStack":
        """
        Create a new explorer map targeting the closest nearby structure of a
        given StructureType.
        
        This method uses implementation default values for radius and
        findUnexplored (usually 100, True).

        Arguments
        - world: the world the map will belong to
        - location: the origin location to find the nearest structure
        - structureType: the type of structure to find
        - radius: radius to search, see World#locateNearestStructure for more
                      information
        - findUnexplored: whether to find unexplored structures

        Returns
        - the newly created item stack

        See
        - World.locateNearestStructure(org.bukkit.Location,
             org.bukkit.StructureType, int, boolean)
        """
        ...


    @staticmethod
    def reload() -> None:
        """
        Reloads the server, refreshing settings and plugin information.
        """
        ...


    @staticmethod
    def reloadData() -> None:
        """
        Reload only the Minecraft data for the server. This includes custom
        advancements and loot tables.
        """
        ...


    @staticmethod
    def getLogger() -> "Logger":
        """
        Returns the primary logger associated with this server instance.

        Returns
        - Logger associated with this server
        """
        ...


    @staticmethod
    def getPluginCommand(name: str) -> "PluginCommand":
        """
        Gets a PluginCommand with the given name or alias.

        Arguments
        - name: the name of the command to retrieve

        Returns
        - a plugin command if found, null otherwise
        """
        ...


    @staticmethod
    def savePlayers() -> None:
        """
        Writes loaded players to disk.
        """
        ...


    @staticmethod
    def dispatchCommand(sender: "CommandSender", commandLine: str) -> bool:
        """
        Dispatches a command on this server, and executes it if found.

        Arguments
        - sender: the apparent sender of the command
        - commandLine: the command + arguments. Example: `test abc
            123`

        Returns
        - returns False if no target is found

        Raises
        - CommandException: thrown when the executor for the given command
            fails with an unhandled exception
        """
        ...


    @staticmethod
    def addRecipe(recipe: "Recipe") -> bool:
        """
        Adds a recipe to the crafting manager.

        Arguments
        - recipe: the recipe to add

        Returns
        - True if the recipe was added, False if it wasn't for some
            reason
        """
        ...


    @staticmethod
    def getRecipesFor(result: "ItemStack") -> list["Recipe"]:
        """
        Get a list of all recipes for a given item. The stack size is ignored
        in comparisons. If the durability is -1, it will match any data value.

        Arguments
        - result: the item to match against recipe results

        Returns
        - a list of recipes with the given result
        """
        ...


    @staticmethod
    def getRecipe(recipeKey: "NamespacedKey") -> "Recipe":
        """
        Get the Recipe for the given key.

        Arguments
        - recipeKey: the key of the recipe to return

        Returns
        - the recipe for the given key or null.
        """
        ...


    @staticmethod
    def getCraftingRecipe(craftingMatrix: list["ItemStack"], world: "World") -> "Recipe":
        """
        Get the Recipe for the list of ItemStacks provided.
        
        The list is formatted as a crafting matrix where the index follow
        the pattern below:
        
        ```
        [ 0 1 2 ]
        [ 3 4 5 ]
        [ 6 7 8 ]
        ```
        
        NOTE: This method will not modify the provided ItemStack array, for that, use
        .craftItem(ItemStack[], World, Player).

        Arguments
        - craftingMatrix: list of items to be crafted from.
                              Must not contain more than 9 items.
        - world: The world the crafting takes place in.

        Returns
        - the Recipe resulting from the given crafting matrix.
        """
        ...


    @staticmethod
    def craftItemResult(craftingMatrix: list["ItemStack"], world: "World", player: "Player") -> "ItemCraftResult":
        """
        Get the crafted item using the list of ItemStack provided.
        
        The list is formatted as a crafting matrix where the index follow
        the pattern below:
        
        ```
        [ 0 1 2 ]
        [ 3 4 5 ]
        [ 6 7 8 ]
        ```
        
        The World and Player arguments are required to fulfill the Bukkit Crafting
        events.
        
        Calls org.bukkit.event.inventory.PrepareItemCraftEvent to imitate the Player
        initiating the crafting event.

        Arguments
        - craftingMatrix: list of items to be crafted from.
                              Must not contain more than 9 items.
        - world: The world the crafting takes place in.
        - player: The player to imitate the crafting event on.

        Returns
        - resulting ItemCraftResult containing the resulting item, matrix and any overflow items.
        """
        ...


    @staticmethod
    def craftItemResult(craftingMatrix: list["ItemStack"], world: "World") -> "ItemCraftResult":
        """
        Get the crafted item using the list of ItemStack provided.
        
        The list is formatted as a crafting matrix where the index follow
        the pattern below:
        
        ```
        [ 0 1 2 ]
        [ 3 4 5 ]
        [ 6 7 8 ]
        ```

        Arguments
        - craftingMatrix: list of items to be crafted from.
                              Must not contain more than 9 items.
        - world: The world the crafting takes place in.

        Returns
        - resulting ItemCraftResult containing the resulting item, matrix and any overflow items.
        """
        ...


    @staticmethod
    def craftItem(craftingMatrix: list["ItemStack"], world: "World", player: "Player") -> "ItemStack":
        """
        Get the crafted item using the list of ItemStack provided.
        
        The list is formatted as a crafting matrix where the index follow
        the pattern below:
        
        ```
        [ 0 1 2 ]
        [ 3 4 5 ]
        [ 6 7 8 ]
        ```
        
        The World and Player arguments are required to fulfill the Bukkit Crafting
        events.
        
        Calls org.bukkit.event.inventory.PrepareItemCraftEvent to imitate the Player
        initiating the crafting event.

        Arguments
        - craftingMatrix: list of items to be crafted from.
                              Must not contain more than 9 items.
        - world: The world the crafting takes place in.
        - player: The player to imitate the crafting event on.

        Returns
        - the ItemStack resulting from the given crafting matrix, if no recipe is found
        an ItemStack of Material.AIR is returned.
        """
        ...


    @staticmethod
    def craftItem(craftingMatrix: list["ItemStack"], world: "World") -> "ItemStack":
        """
        Get the crafted item using the list of ItemStack provided.
        
        The list is formatted as a crafting matrix where the index follow
        the pattern below:
        
        ```
        [ 0 1 2 ]
        [ 3 4 5 ]
        [ 6 7 8 ]
        ```

        Arguments
        - craftingMatrix: list of items to be crafted from.
                              Must not contain more than 9 items.
        - world: The world the crafting takes place in.

        Returns
        - the ItemStack resulting from the given crafting matrix, if no recipe is found
        an ItemStack of Material.AIR is returned.
        """
        ...


    @staticmethod
    def recipeIterator() -> Iterator["Recipe"]:
        """
        Get an iterator through the list of crafting recipes.

        Returns
        - an iterator
        """
        ...


    @staticmethod
    def clearRecipes() -> None:
        """
        Clears the list of crafting recipes.
        """
        ...


    @staticmethod
    def resetRecipes() -> None:
        """
        Resets the list of crafting recipes to the default.
        """
        ...


    @staticmethod
    def removeRecipe(key: "NamespacedKey") -> bool:
        """
        Remove a recipe from the server.
        
        **Note that removing a recipe may cause permanent loss of data
        associated with that recipe (eg whether it has been discovered by
        players).**

        Arguments
        - key: NamespacedKey of recipe to remove.

        Returns
        - True if recipe was removed
        """
        ...


    @staticmethod
    def getCommandAliases() -> dict[str, list[str]]:
        """
        Gets a list of command aliases defined in the server properties.

        Returns
        - a map of aliases to command names
        """
        ...


    @staticmethod
    def getSpawnRadius() -> int:
        """
        Gets the radius, in blocks, around each worlds spawn point to protect.

        Returns
        - spawn radius, or 0 if none
        """
        ...


    @staticmethod
    def setSpawnRadius(value: int) -> None:
        """
        Sets the radius, in blocks, around each worlds spawn point to protect.

        Arguments
        - value: new spawn radius, or 0 if none
        """
        ...


    @staticmethod
    def shouldSendChatPreviews() -> bool:
        """
        Gets whether the server should send a preview of the player's chat
        message to the client when the player sends a message

        Returns
        - True if the server should send a preview, False otherwise

        Deprecated
        - chat previews have been removed
        """
        ...


    @staticmethod
    def isEnforcingSecureProfiles() -> bool:
        """
        Gets whether the server only allow players with Mojang-signed public key
        to join

        Returns
        - True if only Mojang-signed players can join, False otherwise
        """
        ...


    @staticmethod
    def isAcceptingTransfers() -> bool:
        """
        Gets whether this server is allowing connections transferred from other
        servers.

        Returns
        - True if the server accepts transfers, False otherwise
        """
        ...


    @staticmethod
    def getHideOnlinePlayers() -> bool:
        """
        Gets whether the Server hide online players in server status.

        Returns
        - True if the server hide online players, False otherwise
        """
        ...


    @staticmethod
    def getOnlineMode() -> bool:
        """
        Gets whether the Server is in online mode or not.

        Returns
        - True if the server authenticates clients, False otherwise
        """
        ...


    @staticmethod
    def getAllowFlight() -> bool:
        """
        Gets whether this server allows flying or not.

        Returns
        - True if the server allows flight, False otherwise
        """
        ...


    @staticmethod
    def isHardcore() -> bool:
        """
        Gets whether the server is in hardcore mode or not.

        Returns
        - True if the server mode is hardcore, False otherwise
        """
        ...


    @staticmethod
    def shutdown() -> None:
        """
        Shutdowns the server, stopping everything.
        """
        ...


    @staticmethod
    def broadcast(message: str, permission: str) -> int:
        """
        Broadcasts the specified message to every user with the given
        permission name.

        Arguments
        - message: message to broadcast
        - permission: the required permission Permissible
            permissibles must have to receive the broadcast

        Returns
        - number of message recipients
        """
        ...


    @staticmethod
    def getOfflinePlayer(name: str) -> "OfflinePlayer":
        """
        Gets the player by the given name, regardless if they are offline or
        online.
        
        This method may involve a blocking web request to get the UUID for the
        given name.
        
        This will return an object even if the player does not exist. To this
        method, all players will exist.

        Arguments
        - name: the name the player to retrieve

        Returns
        - an offline player

        See
        - .getOfflinePlayer(java.util.UUID)

        Deprecated
        - Persistent storage of users should be by UUID as names are no longer
                    unique past a single session.
        """
        ...


    @staticmethod
    def getOfflinePlayer(id: "UUID") -> "OfflinePlayer":
        """
        Gets the player by the given UUID, regardless if they are offline or
        online.
        
        This will return an object even if the player does not exist. To this
        method, all players will exist.

        Arguments
        - id: the UUID of the player to retrieve

        Returns
        - an offline player
        """
        ...


    @staticmethod
    def createPlayerProfile(uniqueId: "UUID", name: str) -> "PlayerProfile":
        """
        Creates a new PlayerProfile.

        Arguments
        - uniqueId: the unique id
        - name: the name

        Returns
        - the new PlayerProfile

        Raises
        - IllegalArgumentException: if both the unique id is
        `null` and the name is `null` or blank
        """
        ...


    @staticmethod
    def createPlayerProfile(uniqueId: "UUID") -> "PlayerProfile":
        """
        Creates a new PlayerProfile.

        Arguments
        - uniqueId: the unique id

        Returns
        - the new PlayerProfile

        Raises
        - IllegalArgumentException: if the unique id is `null`
        """
        ...


    @staticmethod
    def createPlayerProfile(name: str) -> "PlayerProfile":
        """
        Creates a new PlayerProfile.

        Arguments
        - name: the name

        Returns
        - the new PlayerProfile

        Raises
        - IllegalArgumentException: if the name is `null` or
        blank
        """
        ...


    @staticmethod
    def getIPBans() -> set[str]:
        """
        Gets a set containing all current IPs that are banned.

        Returns
        - a set containing banned IP addresses
        """
        ...


    @staticmethod
    def banIP(address: str) -> None:
        """
        Bans the specified address from the server.

        Arguments
        - address: the IP address to ban

        Deprecated
        - see .banIP(InetAddress)
        """
        ...


    @staticmethod
    def unbanIP(address: str) -> None:
        """
        Unbans the specified address from the server.

        Arguments
        - address: the IP address to unban

        Deprecated
        - see .unbanIP(InetAddress)
        """
        ...


    @staticmethod
    def banIP(address: "InetAddress") -> None:
        """
        Bans the specified address from the server.

        Arguments
        - address: the IP address to ban
        """
        ...


    @staticmethod
    def unbanIP(address: "InetAddress") -> None:
        """
        Unbans the specified address from the server.

        Arguments
        - address: the IP address to unban
        """
        ...


    @staticmethod
    def getBannedPlayers() -> set["OfflinePlayer"]:
        """
        Gets a set containing all banned players.

        Returns
        - a set containing banned players
        """
        ...


    @staticmethod
    def getBanList(type: "BanList.Type") -> "T":
        """
        Gets a ban list for the supplied type.
        
        Type `<T>`: The ban target

        Arguments
        - type: the type of list to fetch, cannot be null

        Returns
        - a ban list of the specified type
        """
        ...


    @staticmethod
    def getOperators() -> set["OfflinePlayer"]:
        """
        Gets a set containing all player operators.

        Returns
        - a set containing player operators
        """
        ...


    @staticmethod
    def getDefaultGameMode() -> "GameMode":
        """
        Gets the default GameMode for new players.

        Returns
        - the default game mode
        """
        ...


    @staticmethod
    def setDefaultGameMode(mode: "GameMode") -> None:
        """
        Sets the default GameMode for new players.

        Arguments
        - mode: the new game mode
        """
        ...


    @staticmethod
    def getConsoleSender() -> "ConsoleCommandSender":
        """
        Gets a ConsoleCommandSender that may be used as an input source
        for this server.

        Returns
        - a console command sender
        """
        ...


    @staticmethod
    def getWorldContainer() -> "File":
        """
        Gets the folder that contains all of the various Worlds.

        Returns
        - folder that contains all worlds
        """
        ...


    @staticmethod
    def getOfflinePlayers() -> list["OfflinePlayer"]:
        """
        Gets every player that has ever played on this server.

        Returns
        - an array containing all previous players
        """
        ...


    @staticmethod
    def getMessenger() -> "Messenger":
        """
        Gets the Messenger responsible for this server.

        Returns
        - messenger responsible for this server
        """
        ...


    @staticmethod
    def getHelpMap() -> "HelpMap":
        """
        Gets the HelpMap providing help topics for this server.

        Returns
        - a help map for this server
        """
        ...


    @staticmethod
    def createInventory(owner: "InventoryHolder", type: "InventoryType") -> "Inventory":
        """
        Creates an empty inventory with the specified type. If the type
        is InventoryType.CHEST, the new inventory has a size of 27;
        otherwise the new inventory has the normal size for its type.
        
        InventoryType.WORKBENCH will not process crafting recipes if
        created with this method. Use
        Player.openWorkbench(Location, boolean) instead.
        
        InventoryType.ENCHANTING will not process ItemStacks
        for possible enchanting results. Use
        Player.openEnchanting(Location, boolean) instead.

        Arguments
        - owner: the holder of the inventory, or null to indicate no holder
        - type: the type of inventory to create

        Returns
        - a new inventory

        Raises
        - IllegalArgumentException: if the InventoryType cannot be
        viewed.

        See
        - InventoryType.isCreatable()
        """
        ...


    @staticmethod
    def createInventory(owner: "InventoryHolder", type: "InventoryType", title: str) -> "Inventory":
        """
        Creates an empty inventory with the specified type and title. If the type
        is InventoryType.CHEST, the new inventory has a size of 27;
        otherwise the new inventory has the normal size for its type.
        It should be noted that some inventory types do not support titles and
        may not render with said titles on the Minecraft client.
        
        InventoryType.WORKBENCH will not process crafting recipes if
        created with this method. Use
        Player.openWorkbench(Location, boolean) instead.
        
        InventoryType.ENCHANTING will not process ItemStacks
        for possible enchanting results. Use
        Player.openEnchanting(Location, boolean) instead.

        Arguments
        - owner: The holder of the inventory; can be null if there's no holder.
        - type: The type of inventory to create.
        - title: The title of the inventory, to be displayed when it is viewed.

        Returns
        - The new inventory.

        Raises
        - IllegalArgumentException: if the InventoryType cannot be
        viewed.

        See
        - InventoryType.isCreatable()
        """
        ...


    @staticmethod
    def createInventory(owner: "InventoryHolder", size: int) -> "Inventory":
        """
        Creates an empty inventory of type InventoryType.CHEST with the
        specified size.

        Arguments
        - owner: the holder of the inventory, or null to indicate no holder
        - size: a multiple of 9 as the size of inventory to create

        Returns
        - a new inventory

        Raises
        - IllegalArgumentException: if the size is not a multiple of 9
        """
        ...


    @staticmethod
    def createInventory(owner: "InventoryHolder", size: int, title: str) -> "Inventory":
        """
        Creates an empty inventory of type InventoryType.CHEST with the
        specified size and title.

        Arguments
        - owner: the holder of the inventory, or null to indicate no holder
        - size: a multiple of 9 as the size of inventory to create
        - title: the title of the inventory, displayed when inventory is
            viewed

        Returns
        - a new inventory

        Raises
        - IllegalArgumentException: if the size is not a multiple of 9
        """
        ...


    @staticmethod
    def createMerchant(title: str) -> "Merchant":
        """
        Creates an empty merchant.

        Arguments
        - title: the title of the corresponding merchant inventory, displayed
        when the merchant inventory is viewed

        Returns
        - a new merchant

        Deprecated
        - The title parameter is no-longer needed when used with
        MenuType.MERCHANT and MenuType.Typed.builder().
        """
        ...


    @staticmethod
    def createMerchant() -> "Merchant":
        """
        Creates an empty merchant.

        Returns
        - a new merchant
        """
        ...


    @staticmethod
    def getMaxChainedNeighborUpdates() -> int:
        """
        Gets the amount of consecutive neighbor updates before skipping
        additional ones.

        Returns
        - the amount of consecutive neighbor updates, if the value is
        negative then the limit it's not used
        """
        ...


    @staticmethod
    def getMonsterSpawnLimit() -> int:
        """
        Gets user-specified limit for number of monsters that can spawn in a
        chunk.

        Returns
        - the monster spawn limit

        Deprecated
        - Deprecated in favor of .getSpawnLimit(SpawnCategory)
        """
        ...


    @staticmethod
    def getAnimalSpawnLimit() -> int:
        """
        Gets user-specified limit for number of animals that can spawn in a
        chunk.

        Returns
        - the animal spawn limit

        Deprecated
        - Deprecated in favor of .getSpawnLimit(SpawnCategory)
        """
        ...


    @staticmethod
    def getWaterAnimalSpawnLimit() -> int:
        """
        Gets user-specified limit for number of water animals that can spawn in
        a chunk.

        Returns
        - the water animal spawn limit

        Deprecated
        - Deprecated in favor of .getSpawnLimit(SpawnCategory)
        """
        ...


    @staticmethod
    def getWaterAmbientSpawnLimit() -> int:
        """
        Gets user-specified limit for number of water ambient mobs that can spawn
        in a chunk.

        Returns
        - the water ambient spawn limit

        Deprecated
        - Deprecated in favor of .getSpawnLimit(SpawnCategory)
        """
        ...


    @staticmethod
    def getWaterUndergroundCreatureSpawnLimit() -> int:
        """
        Get user-specified limit for number of water creature underground that can spawn
        in a chunk.

        Returns
        - the water underground creature limit

        Deprecated
        - Deprecated in favor of .getSpawnLimit(SpawnCategory)
        """
        ...


    @staticmethod
    def getAmbientSpawnLimit() -> int:
        """
        Gets user-specified limit for number of ambient mobs that can spawn in
        a chunk.

        Returns
        - the ambient spawn limit

        Deprecated
        - Deprecated in favor of .getSpawnLimit(SpawnCategory)
        """
        ...


    @staticmethod
    def getSpawnLimit(spawnCategory: "SpawnCategory") -> int:
        """
        Gets user-specified limit for number of SpawnCategory mobs that can spawn in
        a chunk.
        
        **Note: the SpawnCategory.MISC are not consider.**

        Arguments
        - spawnCategory: the category spawn

        Returns
        - the SpawnCategory spawn limit
        """
        ...


    @staticmethod
    def isPrimaryThread() -> bool:
        """
        Checks the current thread against the expected primary thread for the
        server.
        
        **Note:** this method should not be used to indicate the current
        synchronized state of the runtime. A current thread matching the main
        thread indicates that it is synchronized, but a mismatch **does not
        preclude** the same assumption.

        Returns
        - True if the current thread matches the expected primary thread,
            False otherwise
        """
        ...


    @staticmethod
    def getMotd() -> str:
        """
        Gets the message that is displayed on the server list.

        Returns
        - the servers MOTD
        """
        ...


    @staticmethod
    def setMotd(motd: str) -> None:
        """
        Set the message that is displayed on the server list.

        Arguments
        - motd: The message to be displayed
        """
        ...


    @staticmethod
    def getServerLinks() -> "ServerLinks":
        """
        Gets the server links which will be sent to clients

        Returns
        - the server's links
        """
        ...


    @staticmethod
    def getShutdownMessage() -> str:
        """
        Gets the default message that is displayed when the server is stopped.

        Returns
        - the shutdown message
        """
        ...


    @staticmethod
    def getWarningState() -> "WarningState":
        """
        Gets the current warning state for the server.

        Returns
        - the configured warning state
        """
        ...


    @staticmethod
    def getItemFactory() -> "ItemFactory":
        """
        Gets the instance of the item factory (for ItemMeta).

        Returns
        - the item factory

        See
        - ItemFactory
        """
        ...


    @staticmethod
    def getEntityFactory() -> "EntityFactory":
        """
        Gets the instance of the entity factory (for EntitySnapshot).

        Returns
        - the entity factory

        See
        - EntityFactory
        """
        ...


    @staticmethod
    def getScoreboardManager() -> "ScoreboardManager":
        """
        Gets the instance of the scoreboard manager.
        
        This will only exist after the first world has loaded.

        Returns
        - the scoreboard manager or null if no worlds are loaded.
        """
        ...


    @staticmethod
    def getScoreboardCriteria(name: str) -> "Criteria":
        """
        Get (or create) a new Criteria by its name.

        Arguments
        - name: the criteria name

        Returns
        - the criteria

        See
        - Criteria Criteria for a list of constants
        """
        ...


    @staticmethod
    def getServerIcon() -> "CachedServerIcon":
        """
        Gets an instance of the server's default server-icon.

        Returns
        - the default server-icon; null values may be used by the
            implementation to indicate no defined icon, but this behavior is
            not guaranteed
        """
        ...


    @staticmethod
    def loadServerIcon(file: "File") -> "CachedServerIcon":
        """
        Loads an image from a file, and returns a cached image for the specific
        server-icon.
        
        Size and type are implementation defined. An incompatible file is
        guaranteed to throw an implementation-defined Exception.

        Arguments
        - file: the file to load the from

        Returns
        - a cached server-icon that can be used for a ServerListPingEvent.setServerIcon(CachedServerIcon)

        Raises
        - IllegalArgumentException: if image is null
        - Exception: if the image does not meet current server server-icon
            specifications
        """
        ...


    @staticmethod
    def loadServerIcon(image: "BufferedImage") -> "CachedServerIcon":
        """
        Creates a cached server-icon for the specific image.
        
        Size and type are implementation defined. An incompatible file is
        guaranteed to throw an implementation-defined Exception.

        Arguments
        - image: the image to use

        Returns
        - a cached server-icon that can be used for a ServerListPingEvent.setServerIcon(CachedServerIcon)

        Raises
        - IllegalArgumentException: if image is null
        - Exception: if the image does not meet current server
            server-icon specifications
        """
        ...


    @staticmethod
    def setIdleTimeout(threshold: int) -> None:
        """
        Set the idle kick timeout. Any players idle for the specified amount of
        time will be automatically kicked.
        
        A value of 0 will disable the idle kick timeout.

        Arguments
        - threshold: the idle timeout in minutes
        """
        ...


    @staticmethod
    def getIdleTimeout() -> int:
        """
        Gets the idle kick timeout.

        Returns
        - the idle timeout in minutes
        """
        ...


    @staticmethod
    def getPauseWhenEmptyTime() -> int:
        """
        Gets the pause when empty threshold seconds. To save resources, the
        server will pause most functions after this time if there are no players
        online.

        Returns
        - the pause threshold in seconds
        """
        ...


    @staticmethod
    def setPauseWhenEmptyTime(seconds: int) -> None:
        """
        Sets the pause when empty threshold seconds. To save resources, the
        server will pause most functions after this time if there are no players
        online.
        
        A value of less than 0 will disable the setting

        Arguments
        - seconds: the pause threshold in seconds
        """
        ...


    @staticmethod
    def createChunkData(world: "World") -> "ChunkGenerator.ChunkData":
        """
        Create a ChunkData for use in a generator.
        
        See ChunkGenerator.generateChunkData(org.bukkit.World, java.util.Random, int, int, org.bukkit.generator.ChunkGenerator.BiomeGrid)

        Arguments
        - world: the world to create the ChunkData for

        Returns
        - a new ChunkData for the world
        """
        ...


    @staticmethod
    def createBossBar(title: str, color: "BarColor", style: "BarStyle", *flags: Tuple["BarFlag", ...]) -> "BossBar":
        """
        Creates a boss bar instance to display to players. The progress
        defaults to 1.0

        Arguments
        - title: the title of the boss bar
        - color: the color of the boss bar
        - style: the style of the boss bar
        - flags: an optional list of flags to set on the boss bar

        Returns
        - the created boss bar
        """
        ...


    @staticmethod
    def createBossBar(key: "NamespacedKey", title: str, color: "BarColor", style: "BarStyle", *flags: Tuple["BarFlag", ...]) -> "KeyedBossBar":
        """
        Creates a boss bar instance to display to players. The progress defaults
        to 1.0.
        
        This instance is added to the persistent storage of the server and will
        be editable by commands and restored after restart.

        Arguments
        - key: the key of the boss bar that is used to access the boss bar
        - title: the title of the boss bar
        - color: the color of the boss bar
        - style: the style of the boss bar
        - flags: an optional list of flags to set on the boss bar

        Returns
        - the created boss bar
        """
        ...


    @staticmethod
    def getBossBars() -> Iterator["KeyedBossBar"]:
        """
        Gets an unmodifiable iterator through all persistent bossbars.
        
          - **not** bound to a org.bukkit.entity.Boss
          - 
            **not** created using
            .createBossBar(String, BarColor, BarStyle, BarFlag...)
          
        
        
        e.g. bossbars created using the bossbar command

        Returns
        - a bossbar iterator
        """
        ...


    @staticmethod
    def getBossBar(key: "NamespacedKey") -> "KeyedBossBar":
        """
        Gets the KeyedBossBar specified by this key.
        
          - **not** bound to a org.bukkit.entity.Boss
          - 
            **not** created using
            .createBossBar(String, BarColor, BarStyle, BarFlag...)
          
        
        
        e.g. bossbars created using the bossbar command

        Arguments
        - key: unique bossbar key

        Returns
        - bossbar or null if not exists
        """
        ...


    @staticmethod
    def removeBossBar(key: "NamespacedKey") -> bool:
        """
        Removes a KeyedBossBar specified by this key.
        
          - **not** bound to a org.bukkit.entity.Boss
          - 
            **not** created using
            .createBossBar(String, BarColor, BarStyle, BarFlag...)
          
        
        
        e.g. bossbars created using the bossbar command

        Arguments
        - key: unique bossbar key

        Returns
        - True if removal succeeded or False
        """
        ...


    @staticmethod
    def getEntity(uuid: "UUID") -> "Entity":
        """
        Gets an entity on the server by its UUID

        Arguments
        - uuid: the UUID of the entity

        Returns
        - the entity with the given UUID, or null if it isn't found
        """
        ...


    @staticmethod
    def getAdvancement(key: "NamespacedKey") -> "Advancement":
        """
        Get the advancement specified by this key.

        Arguments
        - key: unique advancement key

        Returns
        - advancement or null if not exists
        """
        ...


    @staticmethod
    def advancementIterator() -> Iterator["Advancement"]:
        """
        Get an iterator through all advancements. Advancements cannot be removed
        from this iterator,

        Returns
        - an advancement iterator
        """
        ...


    @staticmethod
    def createBlockData(material: "Material") -> "BlockData":
        """
        Creates a new BlockData instance for the specified Material, with
        all properties initialized to unspecified defaults.

        Arguments
        - material: the material

        Returns
        - new data instance
        """
        ...


    @staticmethod
    def createBlockData(material: "Material", consumer: "Consumer"["BlockData"]) -> "BlockData":
        """
        Creates a new BlockData instance for the specified Material, with
        all properties initialized to unspecified defaults.

        Arguments
        - material: the material
        - consumer: consumer to run on new instance before returning

        Returns
        - new data instance
        """
        ...


    @staticmethod
    def createBlockData(data: str) -> "BlockData":
        """
        Creates a new BlockData instance with material and properties
        parsed from provided data.

        Arguments
        - data: data string

        Returns
        - new data instance

        Raises
        - IllegalArgumentException: if the specified data is not valid
        """
        ...


    @staticmethod
    def createBlockData(material: "Material", data: str) -> "BlockData":
        """
        Creates a new BlockData instance for the specified Material, with
        all properties initialized to unspecified defaults, except for those
        provided in data.

        Arguments
        - material: the material
        - data: data string

        Returns
        - new data instance

        Raises
        - IllegalArgumentException: if the specified data is not valid
        """
        ...


    @staticmethod
    def getTag(registry: str, tag: "NamespacedKey", clazz: type["T"]) -> "Tag"["T"]:
        """
        Gets a tag which has already been defined within the server. Plugins are
        suggested to use the concrete tags in Tag rather than this method
        which makes no guarantees about which tags are available, and may also be
        less performant due to lack of caching.
        
        Tags will be searched for in an implementation specific manner, but a
        path consisting of namespace/tags/registry/key is expected.
        
        Server implementations are allowed to handle only the registries
        indicated in Tag.
        
        Type `<T>`: type of the tag

        Arguments
        - registry: the tag registry to look at
        - tag: the name of the tag
        - clazz: the class of the tag entries

        Returns
        - the tag or null
        """
        ...


    @staticmethod
    def getTags(registry: str, clazz: type["T"]) -> Iterable["Tag"["T"]]:
        """
        Gets a all tags which have been defined within the server.
        
        Server implementations are allowed to handle only the registries
        indicated in Tag.
        
        No guarantees are made about the mutability of the returned iterator.
        
        Type `<T>`: type of the tag

        Arguments
        - registry: the tag registry to look at
        - clazz: the class of the tag entries

        Returns
        - all defined tags
        """
        ...


    @staticmethod
    def getLootTable(key: "NamespacedKey") -> "LootTable":
        """
        Gets the specified LootTable.

        Arguments
        - key: the name of the LootTable

        Returns
        - the LootTable, or null if no LootTable is found with that name
        """
        ...


    @staticmethod
    def selectEntities(sender: "CommandSender", selector: str) -> list["Entity"]:
        """
        Selects entities using the given Vanilla selector.
        
        No guarantees are made about the selector format, other than they match
        the Vanilla format for the active Minecraft version.
        
        Usually a selector will start with '@', unless selecting a Player in
        which case it may simply be the Player's name or UUID.
        
        Note that in Vanilla, elevated permissions are usually required to use
        '@' selectors, but this method should not check such permissions from the
        sender.

        Arguments
        - sender: the sender to execute as, must be provided
        - selector: the selection string

        Returns
        - a list of the selected entities. The list will not be null, but
        no further guarantees are made.

        Raises
        - IllegalArgumentException: if the selector is malformed in any way
        or a parameter is null
        """
        ...


    @staticmethod
    def getStructureManager() -> "StructureManager":
        """
        Gets the structure manager for loading and saving structures.

        Returns
        - the structure manager
        """
        ...


    @staticmethod
    def getRegistry(tClass: type["T"]) -> "Registry"["T"]:
        """
        Returns the registry for the given class.
        
        If no registry is present for the given class null will be returned.
        
        Depending on the implementation not every registry present in
        Registry will be returned by this method.
        
        Type `<T>`: type of the registry

        Arguments
        - tClass: of the registry to get

        Returns
        - the corresponding registry or null if not present
        """
        ...


    @staticmethod
    def getUnsafe() -> "UnsafeValues":
        """
        Returns
        - the unsafe values instance

        See
        - UnsafeValues
        """
        ...


    @staticmethod
    def spigot() -> "Server.Spigot":
        ...



"""
Python module generated from Java source file org.bukkit.attribute.AttributeInstance

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.attribute import *
from typing import Any, Callable, Iterable, Tuple


class AttributeInstance:
    """
    Represents a mutable instance of an attribute and its associated modifiers
    and values.
    """

    def getAttribute(self) -> "Attribute":
        """
        The attribute pertaining to this instance.

        Returns
        - the attribute
        """
        ...


    def getBaseValue(self) -> float:
        """
        Base value of this instance before modifiers are applied.

        Returns
        - base value
        """
        ...


    def setBaseValue(self, value: float) -> None:
        """
        Set the base value of this instance.

        Arguments
        - value: new base value
        """
        ...


    def getModifiers(self) -> Iterable["AttributeModifier"]:
        """
        Get all modifiers present on this instance.

        Returns
        - a copied collection of all modifiers
        """
        ...


    def addModifier(self, modifier: "AttributeModifier") -> None:
        """
        Add a modifier to this instance.

        Arguments
        - modifier: to add
        """
        ...


    def removeModifier(self, modifier: "AttributeModifier") -> None:
        """
        Remove a modifier from this instance.

        Arguments
        - modifier: to remove
        """
        ...


    def getValue(self) -> float:
        """
        Get the value of this instance after all associated modifiers have been
        applied.

        Returns
        - the total attribute value
        """
        ...


    def getDefaultValue(self) -> float:
        """
        Gets the default value of the Attribute attached to this instance.

        Returns
        - server default value
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityResurrectEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import LivingEntity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from org.bukkit.inventory import EquipmentSlot
from typing import Any, Callable, Iterable, Tuple


class EntityResurrectEvent(EntityEvent, Cancellable):
    """
    Called when an entity dies and may have the opportunity to be resurrected.
    Will be called in a cancelled state if the entity does not have a totem
    equipped.
    """

    def __init__(self, what: "LivingEntity", hand: "EquipmentSlot"):
        ...


    def __init__(self, what: "LivingEntity"):
        ...


    def getEntity(self) -> "LivingEntity":
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand in which the totem of undying was found, or null if the
        entity did not have a totem of undying.

        Returns
        - the hand, or null
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ExecutionSequencer

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations.concurrent import LazyInit
from java.util.concurrent import Callable
from java.util.concurrent import Executor
from java.util.concurrent.atomic import AtomicReference
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ExecutionSequencer:
    """
    Serializes execution of tasks, somewhat like an "asynchronous `synchronized` block." Each
    .submit enqueued callable will not be submitted to its associated executor until the
    previous callable has returned -- and, if the previous callable was an AsyncCallable, not
    until the `Future` it returned is Future.isDone done (successful, failed, or
    cancelled).
    
    This class serializes execution of *submitted* tasks but not any *listeners* of
    those tasks.
    
    Submitted tasks have a happens-before order as defined in the Java Language Specification.
    Tasks execute with the same happens-before order that the function calls to .submit and
    .submitAsync that submitted those tasks had.
    
    This class has limited support for cancellation and other "early completions":
    
    
      - While calls to `submit` and `submitAsync` return a `Future` that can be
          cancelled, cancellation never propagates to a task that has started to run -- neither to
          the callable itself nor to any `Future` returned by an `AsyncCallable`.
          (However, cancellation can prevent an *unstarted* task from running.) Therefore, the
          next task will wait for any running callable (or pending `Future` returned by an
          `AsyncCallable`) to complete, without interrupting it (and without calling `cancel` on the `Future`). So beware: *Even if you cancel every preceding `Future` returned by this class, the next task may still have to wait.*.
      - Once an `AsyncCallable` returns a `Future`, this class considers that task to
          be "done" as soon as *that* `Future` completes in any way. Notably, a `Future` is "completed" even if it is cancelled while its underlying work continues on a
          thread, an RPC, etc. The `Future` is also "completed" if it fails "early" -- for
          example, if the deadline expires on a `Future` returned from Futures.withTimeout while the `Future` it wraps continues its underlying work. So
          beware: *Your `AsyncCallable` should not complete its `Future` until it is
          safe for the next task to start.*
    
    
    This class is similar to MoreExecutors.newSequentialExecutor. This class is different
    in a few ways:
    
    
      - Each task may be associated with a different executor.
      - Tasks may be of type `AsyncCallable`.
      - Running tasks *cannot* be interrupted. (Note that `newSequentialExecutor` does
          not return `Future` objects, so it doesn't support interruption directly, either.
          However, utilities that *use* that executor have the ability to interrupt tasks
          running on it. This class, by contrast, does not expose an `Executor` API.)
    
    
    If you don't need the features of this class, you may prefer `newSequentialExecutor` for
    its simplicity and ability to accommodate interruption.

    Since
    - 26.0
    """

    @staticmethod
    def create() -> "ExecutionSequencer":
        """
        Creates a new instance.
        """
        ...


    def submit(self, callable: "Callable"["T"], executor: "Executor") -> "ListenableFuture"["T"]:
        """
        Enqueues a task to run when the previous task (if any) completes.
        
        Cancellation does not propagate from the output future to a callable that has begun to
        execute, but if the output future is cancelled before Callable.call() is invoked,
        Callable.call() will not be invoked.
        """
        ...


    def submitAsync(self, callable: "AsyncCallable"["T"], executor: "Executor") -> "ListenableFuture"["T"]:
        """
        Enqueues a task to run when the previous task (if any) completes.
        
        Cancellation does not propagate from the output future to the future returned from `callable` or a callable that has begun to execute, but if the output future is cancelled before
        AsyncCallable.call() is invoked, AsyncCallable.call() will not be invoked.
        """
        ...



from .VillagerCareerChangeEvent import VillagerCareerChangeEvent
from .EntityPortalExitEvent import EntityPortalExitEvent
from .EntityTeleportEvent import EntityTeleportEvent
from .ItemSpawnEvent import ItemSpawnEvent
from .EntityBreedEvent import EntityBreedEvent
from .PigZombieAngerEvent import PigZombieAngerEvent
from .EntityToggleGlideEvent import EntityToggleGlideEvent
from .AreaEffectCloudApplyEvent import AreaEffectCloudApplyEvent
from .HorseJumpEvent import HorseJumpEvent
from .EntityTargetEvent import EntityTargetEvent
from .EntityKnockbackByEntityEvent import EntityKnockbackByEntityEvent
from .EntityEvent import EntityEvent
from .ProjectileLaunchEvent import ProjectileLaunchEvent
from .EntityPoseChangeEvent import EntityPoseChangeEvent
from .SlimeSplitEvent import SlimeSplitEvent
from .FireworkExplodeEvent import FireworkExplodeEvent
from .EntityDamageByBlockEvent import EntityDamageByBlockEvent
from .ArrowBodyCountChangeEvent import ArrowBodyCountChangeEvent
from .FoodLevelChangeEvent import FoodLevelChangeEvent
from .EntityExplodeEvent import EntityExplodeEvent
from .SpawnerSpawnEvent import SpawnerSpawnEvent
from .EntityBreakDoorEvent import EntityBreakDoorEvent
from .TrialSpawnerSpawnEvent import TrialSpawnerSpawnEvent
from .ItemMergeEvent import ItemMergeEvent
from .EntityCombustByBlockEvent import EntityCombustByBlockEvent
from .ProjectileHitEvent import ProjectileHitEvent
from .EntitySpawnEvent import EntitySpawnEvent
from .EntityPotionEffectEvent import EntityPotionEffectEvent
from .EntityCombustByEntityEvent import EntityCombustByEntityEvent
from .VillagerReplenishTradeEvent import VillagerReplenishTradeEvent
from .StriderTemperatureChangeEvent import StriderTemperatureChangeEvent
from .EntityDamageEvent import EntityDamageEvent
from .EntityTameEvent import EntityTameEvent
from .EntityInteractEvent import EntityInteractEvent
from .EntityPickupItemEvent import EntityPickupItemEvent
from .EntitySpellCastEvent import EntitySpellCastEvent
from .EntityDamageByEntityEvent import EntityDamageByEntityEvent
from .EntityKnockbackEvent import EntityKnockbackEvent
from .ExpBottleEvent import ExpBottleEvent
from .EntityRegainHealthEvent import EntityRegainHealthEvent
from .EntityPortalEvent import EntityPortalEvent
from .BatToggleSleepEvent import BatToggleSleepEvent
from .EntityExhaustionEvent import EntityExhaustionEvent
from .EntityAirChangeEvent import EntityAirChangeEvent
from .PigZapEvent import PigZapEvent
from .EntityPortalEnterEvent import EntityPortalEnterEvent
from .EntityDeathEvent import EntityDeathEvent
from .EntityUnleashEvent import EntityUnleashEvent
from .VillagerReputationChangeEvent import VillagerReputationChangeEvent
from .EntityRemoveEvent import EntityRemoveEvent
from .EntityResurrectEvent import EntityResurrectEvent
from .EntityMountEvent import EntityMountEvent
from .EntityCombustEvent import EntityCombustEvent
from .SheepDyeWoolEvent import SheepDyeWoolEvent
from .ItemDespawnEvent import ItemDespawnEvent
from .EntityDropItemEvent import EntityDropItemEvent
from .EntityPlaceEvent import EntityPlaceEvent
from .EntityCreatePortalEvent import EntityCreatePortalEvent
from .EntityEnterBlockEvent import EntityEnterBlockEvent
from .ExplosionPrimeEvent import ExplosionPrimeEvent
from .PotionSplashEvent import PotionSplashEvent
from .EntityEnterLoveModeEvent import EntityEnterLoveModeEvent
from .EntityTransformEvent import EntityTransformEvent
from .PlayerDeathEvent import PlayerDeathEvent
from .CreatureSpawnEvent import CreatureSpawnEvent
from .EntityTargetLivingEntityEvent import EntityTargetLivingEntityEvent
from .VillagerAcquireTradeEvent import VillagerAcquireTradeEvent
from .CreeperPowerEvent import CreeperPowerEvent
from .EnderDragonChangePhaseEvent import EnderDragonChangePhaseEvent
from .PiglinBarterEvent import PiglinBarterEvent
from .PlayerLeashEntityEvent import PlayerLeashEntityEvent
from .LingeringPotionSplashEvent import LingeringPotionSplashEvent
from .EntityShootBowEvent import EntityShootBowEvent
from .EntityDismountEvent import EntityDismountEvent
from .EntityChangeBlockEvent import EntityChangeBlockEvent
from .EntityToggleSwimEvent import EntityToggleSwimEvent
from .SheepRegrowWoolEvent import SheepRegrowWoolEvent


"""
Python module generated from Java source file org.bukkit.util.io.BukkitObjectOutputStream

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import ObjectOutputStream
from java.io import OutputStream
from java.io import Serializable
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.util.io import *
from typing import Any, Callable, Iterable, Tuple


class BukkitObjectOutputStream(ObjectOutputStream):
    """
    This class is designed to be used in conjunction with the ConfigurationSerializable API. It translates objects to an internal
    implementation for later deserialization using BukkitObjectInputStream.
    
    Behavior of implementations extending this class is not guaranteed across
    future versions.
    """

    def __init__(self, out: "OutputStream"):
        """
        Object output stream decoration constructor.

        Arguments
        - out: the stream to wrap

        Raises
        - IOException: if an I/O error occurs while writing stream header

        See
        - ObjectOutputStream.ObjectOutputStream(OutputStream)
        """
        ...



"""
Python module generated from Java source file java.lang.ref.PhantomReference

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.ref import *
from jdk.internal.vm.annotation import IntrinsicCandidate
from typing import Any, Callable, Iterable, Tuple


class PhantomReference(Reference):

    def __init__(self, referent: "T", q: "ReferenceQueue"["T"]):
        """
        Creates a new phantom reference that refers to the given object and
        is registered with the given queue.
        
         It is possible to create a phantom reference with a `null`
        queue.  Such a reference will never be enqueued.

        Arguments
        - referent: the object the new phantom reference will refer to
        - q: the queue with which the reference is to be registered,
                 or `null` if registration is not required
        """
        ...


    def get(self) -> "T":
        """
        Returns this reference object's referent.  Because the referent of a
        phantom reference is always inaccessible, this method always returns
        `null`.

        Returns
        - `null`
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Colorable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import DyeColor
from org.bukkit import UndefinedNullability
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Colorable:
    """
    An object that can be colored.
    """

    def getColor(self) -> "DyeColor":
        """
        Gets the color of this object.
        
        This may be null to represent the default color of an object, if the
        object has a special default color (e.g Shulkers).

        Returns
        - The DyeColor of this object.
        """
        ...


    def setColor(self, color: "DyeColor") -> None:
        """
        Sets the color of this object to the specified DyeColor.
        
        This may be null to represent the default color of an object, if the
        object has a special default color (e.g Shulkers).

        Arguments
        - color: The color of the object, as a DyeColor.

        Raises
        - NullPointerException: if argument is null and this implementation does not support null
        """
        ...



"""
Python module generated from Java source file com.google.common.io.LineProcessor

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import IOException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class LineProcessor:
    """
    A callback to be used with the streaming `readLines` methods.
    
    .processLine will be called for each line that is read, and should return `False` when you want to stop processing.

    Author(s)
    - Miles Barr

    Since
    - 1.0
    """

    def processLine(self, line: str) -> bool:
        """
        This method will be called once for each line.

        Arguments
        - line: the line read from the input, without delimiter

        Returns
        - True to continue processing, False to stop
        """
        ...


    def getResult(self) -> "T":
        """
        Return the result of processing all the lines.
        """
        ...



"""
Python module generated from Java source file org.bukkit.Location

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.lang.ref import Reference
from java.lang.ref import WeakReference
from org.bukkit import *
from org.bukkit.block import Block
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.util import NumberConversions
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class Location(Cloneable, ConfigurationSerializable):
    """
    Represents a 3-dimensional position in a world.
    
    No constraints are placed on any angular values other than that they be
    specified in degrees. This means that negative angles or angles of greater
    magnitude than 360 are valid, but may be normalized to any other equivalent
    representation by the implementation.
    """

    def __init__(self, world: "World", x: float, y: float, z: float):
        """
        Constructs a new Location with the given coordinates

        Arguments
        - world: The world in which this location resides
        - x: The x-coordinate of this new location
        - y: The y-coordinate of this new location
        - z: The z-coordinate of this new location
        """
        ...


    def __init__(self, world: "World", x: float, y: float, z: float, yaw: float, pitch: float):
        """
        Constructs a new Location with the given coordinates and direction

        Arguments
        - world: The world in which this location resides
        - x: The x-coordinate of this new location
        - y: The y-coordinate of this new location
        - z: The z-coordinate of this new location
        - yaw: The absolute rotation on the x-plane, in degrees
        - pitch: The absolute rotation on the y-plane, in degrees
        """
        ...


    def setWorld(self, world: "World") -> None:
        """
        Sets the world that this location resides in

        Arguments
        - world: New world that this location resides in
        """
        ...


    def isWorldLoaded(self) -> bool:
        """
        Checks if world in this location is present and loaded.

        Returns
        - True if is loaded, otherwise False
        """
        ...


    def getWorld(self) -> "World":
        """
        Gets the world that this location resides in

        Returns
        - World that contains this location, or `null` if it is not set

        Raises
        - IllegalArgumentException: when world is unloaded

        See
        - .isWorldLoaded()
        """
        ...


    def getChunk(self) -> "Chunk":
        """
        Gets the chunk at the represented location

        Returns
        - Chunk at the represented location
        """
        ...


    def getBlock(self) -> "Block":
        """
        Gets the block at the represented location

        Returns
        - Block at the represented location
        """
        ...


    def setX(self, x: float) -> None:
        """
        Sets the x-coordinate of this location

        Arguments
        - x: X-coordinate
        """
        ...


    def getX(self) -> float:
        """
        Gets the x-coordinate of this location

        Returns
        - x-coordinate
        """
        ...


    def getBlockX(self) -> int:
        """
        Gets the floored value of the X component, indicating the block that
        this location is contained with.

        Returns
        - block X
        """
        ...


    def setY(self, y: float) -> None:
        """
        Sets the y-coordinate of this location

        Arguments
        - y: y-coordinate
        """
        ...


    def getY(self) -> float:
        """
        Gets the y-coordinate of this location

        Returns
        - y-coordinate
        """
        ...


    def getBlockY(self) -> int:
        """
        Gets the floored value of the Y component, indicating the block that
        this location is contained with.

        Returns
        - block y
        """
        ...


    def setZ(self, z: float) -> None:
        """
        Sets the z-coordinate of this location

        Arguments
        - z: z-coordinate
        """
        ...


    def getZ(self) -> float:
        """
        Gets the z-coordinate of this location

        Returns
        - z-coordinate
        """
        ...


    def getBlockZ(self) -> int:
        """
        Gets the floored value of the Z component, indicating the block that
        this location is contained with.

        Returns
        - block z
        """
        ...


    def setYaw(self, yaw: float) -> None:
        """
        Sets the yaw of this location, measured in degrees.
        
        - A yaw of 0 or 360 represents the positive z direction.
        - A yaw of 180 represents the negative z direction.
        - A yaw of 90 represents the negative x direction.
        - A yaw of 270 represents the positive x direction.
        
        Increasing yaw values are the equivalent of turning to your
        right-facing, increasing the scale of the next respective axis, and
        decreasing the scale of the previous axis.

        Arguments
        - yaw: new rotation's yaw
        """
        ...


    def getYaw(self) -> float:
        """
        Gets the yaw of this location, measured in degrees.
        
        - A yaw of 0 or 360 represents the positive z direction.
        - A yaw of 180 represents the negative z direction.
        - A yaw of 90 represents the negative x direction.
        - A yaw of 270 represents the positive x direction.
        
        Increasing yaw values are the equivalent of turning to your
        right-facing, increasing the scale of the next respective axis, and
        decreasing the scale of the previous axis.

        Returns
        - the rotation's yaw
        """
        ...


    def setPitch(self, pitch: float) -> None:
        """
        Sets the pitch of this location, measured in degrees.
        
        - A pitch of 0 represents level forward facing.
        - A pitch of 90 represents downward facing, or negative y
            direction.
        - A pitch of -90 represents upward facing, or positive y direction.
        
        Increasing pitch values the equivalent of looking down.

        Arguments
        - pitch: new incline's pitch
        """
        ...


    def getPitch(self) -> float:
        """
        Gets the pitch of this location, measured in degrees.
        
        - A pitch of 0 represents level forward facing.
        - A pitch of 90 represents downward facing, or negative y
            direction.
        - A pitch of -90 represents upward facing, or positive y direction.
        
        Increasing pitch values the equivalent of looking down.

        Returns
        - the incline's pitch
        """
        ...


    def getDirection(self) -> "Vector":
        """
        Gets a unit-vector pointing in the direction that this Location is
        facing.

        Returns
        - a vector pointing the direction of this location's .getPitch() pitch and .getYaw() yaw
        """
        ...


    def setDirection(self, vector: "Vector") -> "Location":
        """
        Sets the .getYaw() yaw and .getPitch() pitch to point
        in the direction of the vector.

        Arguments
        - vector: the direction vector

        Returns
        - the same location
        """
        ...


    def add(self, vec: "Location") -> "Location":
        """
        Adds the location by another.

        Arguments
        - vec: The other location

        Returns
        - the same location

        Raises
        - IllegalArgumentException: for differing worlds

        See
        - Vector
        """
        ...


    def add(self, vec: "Vector") -> "Location":
        """
        Adds the location by a vector.

        Arguments
        - vec: Vector to use

        Returns
        - the same location

        See
        - Vector
        """
        ...


    def add(self, x: float, y: float, z: float) -> "Location":
        """
        Adds the location by another. Not world-aware.

        Arguments
        - x: X coordinate
        - y: Y coordinate
        - z: Z coordinate

        Returns
        - the same location

        See
        - Vector
        """
        ...


    def subtract(self, vec: "Location") -> "Location":
        """
        Subtracts the location by another.

        Arguments
        - vec: The other location

        Returns
        - the same location

        Raises
        - IllegalArgumentException: for differing worlds

        See
        - Vector
        """
        ...


    def subtract(self, vec: "Vector") -> "Location":
        """
        Subtracts the location by a vector.

        Arguments
        - vec: The vector to use

        Returns
        - the same location

        See
        - Vector
        """
        ...


    def subtract(self, x: float, y: float, z: float) -> "Location":
        """
        Subtracts the location by another. Not world-aware and
        orientation independent.

        Arguments
        - x: X coordinate
        - y: Y coordinate
        - z: Z coordinate

        Returns
        - the same location

        See
        - Vector
        """
        ...


    def length(self) -> float:
        """
        Gets the magnitude of the location, defined as sqrt(x^2+y^2+z^2). The
        value of this method is not cached and uses a costly square-root
        function, so do not repeatedly call this method to get the location's
        magnitude. NaN will be returned if the inner result of the sqrt()
        function overflows, which will be caused if the length is too long. Not
        world-aware and orientation independent.

        Returns
        - the magnitude

        See
        - Vector
        """
        ...


    def lengthSquared(self) -> float:
        """
        Gets the magnitude of the location squared. Not world-aware and
        orientation independent.

        Returns
        - the magnitude

        See
        - Vector
        """
        ...


    def distance(self, o: "Location") -> float:
        """
        Get the distance between this location and another. The value of this
        method is not cached and uses a costly square-root function, so do not
        repeatedly call this method to get the location's magnitude. NaN will
        be returned if the inner result of the sqrt() function overflows, which
        will be caused if the distance is too long.

        Arguments
        - o: The other location

        Returns
        - the distance

        Raises
        - IllegalArgumentException: for differing worlds

        See
        - Vector
        """
        ...


    def distanceSquared(self, o: "Location") -> float:
        """
        Get the squared distance between this location and another.

        Arguments
        - o: The other location

        Returns
        - the distance

        Raises
        - IllegalArgumentException: for differing worlds

        See
        - Vector
        """
        ...


    def multiply(self, m: float) -> "Location":
        """
        Performs scalar multiplication, multiplying all components with a
        scalar. Not world-aware.

        Arguments
        - m: The factor

        Returns
        - the same location

        See
        - Vector
        """
        ...


    def zero(self) -> "Location":
        """
        Zero this location's components. Not world-aware.

        Returns
        - the same location

        See
        - Vector
        """
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...


    def toVector(self) -> "Vector":
        """
        Constructs a new Vector based on this Location

        Returns
        - New Vector containing the coordinates represented by this
            Location
        """
        ...


    def clone(self) -> "Location":
        ...


    def checkFinite(self) -> None:
        """
        Check if each component of this Location is finite.

        Raises
        - IllegalArgumentException: if any component is not finite
        """
        ...


    @staticmethod
    def locToBlock(loc: float) -> int:
        """
        Safely converts a double (location coordinate) to an int (block
        coordinate)

        Arguments
        - loc: Precise coordinate

        Returns
        - Block coordinate
        """
        ...


    def serialize(self) -> dict[str, "Object"]:
        ...


    @staticmethod
    def deserialize(args: dict[str, "Object"]) -> "Location":
        """
        Required method for deserialization

        Arguments
        - args: map to deserialize

        Returns
        - deserialized location

        Raises
        - IllegalArgumentException: if the world don't exists

        See
        - ConfigurationSerializable
        """
        ...


    @staticmethod
    def normalizeYaw(yaw: float) -> float:
        """
        Normalizes the given yaw angle to a value between `+/-180`
        degrees.

        Arguments
        - yaw: the yaw in degrees

        Returns
        - the normalized yaw in degrees

        See
        - Location.getYaw()
        """
        ...


    @staticmethod
    def normalizePitch(pitch: float) -> float:
        """
        Normalizes the given pitch angle to a value between `+/-90`
        degrees.

        Arguments
        - pitch: the pitch in degrees

        Returns
        - the normalized pitch in degrees

        See
        - Location.getPitch()
        """
        ...



"""
Python module generated from Java source file org.bukkit.NetherWartsState

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class NetherWartsState(Enum):

    SEEDED = 0
    """
    State when first seeded
    """
    STAGE_ONE = 1
    """
    First growth stage
    """
    STAGE_TWO = 2
    """
    Second growth stage
    """
    RIPE = 3
    """
    Ready to harvest
    """



"""
Python module generated from Java source file org.bukkit.CropState

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Maps
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class CropState(Enum):
    """
    Represents the different growth states of crops
    """

    SEEDED = (0x0)
    """
    State when first seeded
    """
    GERMINATED = (0x1)
    """
    First growth stage
    """
    VERY_SMALL = (0x2)
    """
    Second growth stage
    """
    SMALL = (0x3)
    """
    Third growth stage
    """
    MEDIUM = (0x4)
    """
    Fourth growth stage
    """
    TALL = (0x5)
    """
    Fifth growth stage
    """
    VERY_TALL = (0x6)
    """
    Almost ripe stage
    """
    RIPE = (0x7)
    """
    Ripe stage
    """


    def getData(self) -> int:
        """
        Gets the associated data value representing this growth state

        Returns
        - A byte containing the data value of this growth state

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByData(data: int) -> "CropState":
        """
        Gets the CropState with the given data value

        Arguments
        - data: Data value to fetch

        Returns
        - The CropState representing the given value, or null if
            it doesn't exist

        Deprecated
        - Magic value
        """
        ...



"""
Python module generated from Java source file java.net.Inet6Address

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import ObjectStreamField
from java.net import *
from java.util import Arrays
from java.util import Enumeration
from typing import Any, Callable, Iterable, Tuple


class Inet6Address(InetAddress):

    @staticmethod
    def getByAddress(host: str, addr: list[int], nif: "NetworkInterface") -> "Inet6Address":
        """
        Create an Inet6Address in the exact manner of InetAddress.getByAddress(String,byte[]) except that the IPv6 scope_id is
        set to the value corresponding to the given interface for the address
        type specified in `addr`. The call will fail with an
        UnknownHostException if the given interface does not have a numeric
        scope_id assigned for the given address type (e.g. link-local or site-local).
        See <a href="Inet6Address.html#scoped">here</a> for a description of IPv6
        scoped addresses.

        Arguments
        - host: the specified host
        - addr: the raw IP address in network byte order
        - nif: an interface this address must be associated with.

        Returns
        - an Inet6Address object created from the raw IP address.

        Raises
        - UnknownHostException: if IP address is of illegal length, or if the interface does not
                 have a numeric scope_id assigned for the given address type.

        Since
        - 1.5
        """
        ...


    @staticmethod
    def getByAddress(host: str, addr: list[int], scope_id: int) -> "Inet6Address":
        """
        Create an Inet6Address in the exact manner of InetAddress.getByAddress(String,byte[]) except that the IPv6 scope_id is
        set to the given numeric value. The scope_id is not checked to determine
        if it corresponds to any interface on the system.
        See <a href="Inet6Address.html#scoped">here</a> for a description of IPv6
        scoped addresses.

        Arguments
        - host: the specified host
        - addr: the raw IP address in network byte order
        - scope_id: the numeric scope_id for the address.

        Returns
        - an Inet6Address object created from the raw IP address.

        Raises
        - UnknownHostException: if IP address is of illegal length.

        Since
        - 1.5
        """
        ...


    def isMulticastAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is an IP multicast
        address. 11111111 at the start of the address identifies the
        address as being a multicast address.

        Returns
        - a `boolean` indicating if the InetAddress is an IP
                multicast address
        """
        ...


    def isAnyLocalAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is a wildcard address.

        Returns
        - a `boolean` indicating if the InetAddress is
                a wildcard address.
        """
        ...


    def isLoopbackAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is a loopback address.

        Returns
        - a `boolean` indicating if the InetAddress is a loopback
                address; or False otherwise.
        """
        ...


    def isLinkLocalAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is an link local address.

        Returns
        - a `boolean` indicating if the InetAddress is a link local
                address; or False if address is not a link local unicast address.
        """
        ...


    def isSiteLocalAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is a site local address.

        Returns
        - a `boolean` indicating if the InetAddress is a site local
                address; or False if address is not a site local unicast address.
        """
        ...


    def isMCGlobal(self) -> bool:
        """
        Utility routine to check if the multicast address has global scope.

        Returns
        - a `boolean` indicating if the address has is a multicast
                address of global scope, False if it is not of global scope or
                it is not a multicast address
        """
        ...


    def isMCNodeLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has node scope.

        Returns
        - a `boolean` indicating if the address has is a multicast
                address of node-local scope, False if it is not of node-local
                scope or it is not a multicast address
        """
        ...


    def isMCLinkLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has link scope.

        Returns
        - a `boolean` indicating if the address has is a multicast
                address of link-local scope, False if it is not of link-local
                scope or it is not a multicast address
        """
        ...


    def isMCSiteLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has site scope.

        Returns
        - a `boolean` indicating if the address has is a multicast
                address of site-local scope, False if it is not  of site-local
                scope or it is not a multicast address
        """
        ...


    def isMCOrgLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has organization scope.

        Returns
        - a `boolean` indicating if the address has is a multicast
                address of organization-local scope, False if it is not of
                organization-local scope or it is not a multicast address
        """
        ...


    def getAddress(self) -> list[int]:
        """
        Returns the raw IP address of this `InetAddress` object. The result
        is in network byte order: the highest order byte of the address is in
        `getAddress()[0]`.

        Returns
        - the raw IP address of this object.
        """
        ...


    def getScopeId(self) -> int:
        """
        Returns the numeric scopeId, if this instance is associated with
        an interface. If no scoped_id is set, the returned value is zero.

        Returns
        - the scopeId, or zero if not set.

        Since
        - 1.5
        """
        ...


    def getScopedInterface(self) -> "NetworkInterface":
        """
        Returns the scoped interface, if this instance was created with
        a scoped interface.

        Returns
        - the scoped interface, or null if not set.

        Since
        - 1.5
        """
        ...


    def getHostAddress(self) -> str:
        """
        Returns the IP address string in textual presentation. If the instance
        was created specifying a scope identifier then the scope id is appended
        to the IP address preceded by a "%" (per-cent) character. This can be
        either a numeric value or a string, depending on which was used to create
        the instance.

        Returns
        - the raw IP address in a string format.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hashcode for this IP address.

        Returns
        - a hash code value for this IP address.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares this object against the specified object. The result is `True` if and only if the argument is not `null` and it represents
        the same IP address as this object.
        
         Two instances of `InetAddress` represent the same IP address
        if the length of the byte arrays returned by `getAddress` is the
        same for both, and each of the array components is the same for the byte
        arrays.

        Arguments
        - obj: the object to compare against.

        Returns
        - `True` if the objects are the same; `False` otherwise.

        See
        - java.net.InetAddress.getAddress()
        """
        ...


    def isIPv4CompatibleAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is an
        IPv4 compatible IPv6 address.

        Returns
        - a `boolean` indicating if the InetAddress is an IPv4
                compatible IPv6 address; or False if address is IPv4 address.
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.util.ArrayStack

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.util import *
from typing import Any, Callable, Iterable, Tuple


class ArrayStack:
    """
    Custom stack
    
    Type `<T>`: data to keep in stack
    """

    def __init__(self, initSize: int):
        """
        Create

        Arguments
        - initSize: - book the size
        """
        ...


    def push(self, obj: "T") -> None:
        """
        Add the element to the head

        Arguments
        - obj: - data to be added
        """
        ...


    def pop(self) -> "T":
        """
        Get the head and remove it from the stack

        Returns
        - the head
        """
        ...


    def isEmpty(self) -> bool:
        """
        Check

        Returns
        - True when it contains nothing
        """
        ...


    def clear(self) -> None:
        """
        remove all items in the stack
        """
        ...



"""
Python module generated from Java source file java.util.stream.StreamSupport

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Objects
from java.util import Spliterator
from java.util.function import Supplier
from java.util.stream import *
from typing import Any, Callable, Iterable, Tuple


class StreamSupport:
    """
    Low-level utility methods for creating and manipulating streams.
    
    This class is mostly for library writers presenting stream views
    of data structures; most static stream methods intended for end users are in
    the various `Stream` classes.

    Since
    - 1.8
    """

    @staticmethod
    def stream(spliterator: "Spliterator"["T"], parallel: bool) -> "Stream"["T"]:
        """
        Creates a new sequential or parallel `Stream` from a
        `Spliterator`.
        
        The spliterator is only traversed, split, or queried for estimated
        size after the terminal operation of the stream pipeline commences.
        
        It is strongly recommended the spliterator report a characteristic of
        `IMMUTABLE` or `CONCURRENT`, or be
        <a href="../Spliterator.html#binding">late-binding</a>.  Otherwise,
        .stream(java.util.function.Supplier, int, boolean) should be used
        to reduce the scope of potential interference with the source.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.
        
        Type `<T>`: the type of stream elements

        Arguments
        - spliterator: a `Spliterator` describing the stream elements
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `Stream`
        """
        ...


    @staticmethod
    def stream(supplier: "Supplier"["Spliterator"["T"]], characteristics: int, parallel: bool) -> "Stream"["T"]:
        """
        Creates a new sequential or parallel `Stream` from a
        `Supplier` of `Spliterator`.
        
        The Supplier.get() method will be invoked on the supplier no
        more than once, and only after the terminal operation of the stream pipeline
        commences.
        
        For spliterators that report a characteristic of `IMMUTABLE`
        or `CONCURRENT`, or that are
        <a href="../Spliterator.html#binding">late-binding</a>, it is likely
        more efficient to use .stream(java.util.Spliterator, boolean)
        instead.
        The use of a `Supplier` in this form provides a level of
        indirection that reduces the scope of potential interference with the
        source.  Since the supplier is only invoked after the terminal operation
        commences, any modifications to the source up to the start of the
        terminal operation are reflected in the stream result.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.
        
        Type `<T>`: the type of stream elements

        Arguments
        - supplier: a `Supplier` of a `Spliterator`
        - characteristics: Spliterator characteristics of the supplied
               `Spliterator`.  The characteristics must be equal to
               `supplier.get().characteristics()`, otherwise undefined
               behavior may occur when terminal operation commences.
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `Stream`

        See
        - .stream(java.util.Spliterator, boolean)
        """
        ...


    @staticmethod
    def intStream(spliterator: "Spliterator.OfInt", parallel: bool) -> "IntStream":
        """
        Creates a new sequential or parallel `IntStream` from a
        `Spliterator.OfInt`.
        
        The spliterator is only traversed, split, or queried for estimated size
        after the terminal operation of the stream pipeline commences.
        
        It is strongly recommended the spliterator report a characteristic of
        `IMMUTABLE` or `CONCURRENT`, or be
        <a href="../Spliterator.html#binding">late-binding</a>.  Otherwise,
        .intStream(java.util.function.Supplier, int, boolean) should be
        used to reduce the scope of potential interference with the source.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.

        Arguments
        - spliterator: a `Spliterator.OfInt` describing the stream elements
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `IntStream`
        """
        ...


    @staticmethod
    def intStream(supplier: "Supplier"["Spliterator.OfInt"], characteristics: int, parallel: bool) -> "IntStream":
        """
        Creates a new sequential or parallel `IntStream` from a
        `Supplier` of `Spliterator.OfInt`.
        
        The Supplier.get() method will be invoked on the supplier no
        more than once, and only after the terminal operation of the stream pipeline
        commences.
        
        For spliterators that report a characteristic of `IMMUTABLE`
        or `CONCURRENT`, or that are
        <a href="../Spliterator.html#binding">late-binding</a>, it is likely
        more efficient to use .intStream(java.util.Spliterator.OfInt, boolean)
        instead.
        The use of a `Supplier` in this form provides a level of
        indirection that reduces the scope of potential interference with the
        source.  Since the supplier is only invoked after the terminal operation
        commences, any modifications to the source up to the start of the
        terminal operation are reflected in the stream result.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.

        Arguments
        - supplier: a `Supplier` of a `Spliterator.OfInt`
        - characteristics: Spliterator characteristics of the supplied
               `Spliterator.OfInt`.  The characteristics must be equal to
               `supplier.get().characteristics()`, otherwise undefined
               behavior may occur when terminal operation commences.
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `IntStream`

        See
        - .intStream(java.util.Spliterator.OfInt, boolean)
        """
        ...


    @staticmethod
    def longStream(spliterator: "Spliterator.OfLong", parallel: bool) -> "LongStream":
        """
        Creates a new sequential or parallel `LongStream` from a
        `Spliterator.OfLong`.
        
        The spliterator is only traversed, split, or queried for estimated
        size after the terminal operation of the stream pipeline commences.
        
        It is strongly recommended the spliterator report a characteristic of
        `IMMUTABLE` or `CONCURRENT`, or be
        <a href="../Spliterator.html#binding">late-binding</a>.  Otherwise,
        .longStream(java.util.function.Supplier, int, boolean) should be
        used to reduce the scope of potential interference with the source.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.

        Arguments
        - spliterator: a `Spliterator.OfLong` describing the stream elements
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `LongStream`
        """
        ...


    @staticmethod
    def longStream(supplier: "Supplier"["Spliterator.OfLong"], characteristics: int, parallel: bool) -> "LongStream":
        """
        Creates a new sequential or parallel `LongStream` from a
        `Supplier` of `Spliterator.OfLong`.
        
        The Supplier.get() method will be invoked on the supplier no
        more than once, and only after the terminal operation of the stream pipeline
        commences.
        
        For spliterators that report a characteristic of `IMMUTABLE`
        or `CONCURRENT`, or that are
        <a href="../Spliterator.html#binding">late-binding</a>, it is likely
        more efficient to use .longStream(java.util.Spliterator.OfLong, boolean)
        instead.
        The use of a `Supplier` in this form provides a level of
        indirection that reduces the scope of potential interference with the
        source.  Since the supplier is only invoked after the terminal operation
        commences, any modifications to the source up to the start of the
        terminal operation are reflected in the stream result.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.

        Arguments
        - supplier: a `Supplier` of a `Spliterator.OfLong`
        - characteristics: Spliterator characteristics of the supplied
               `Spliterator.OfLong`.  The characteristics must be equal to
               `supplier.get().characteristics()`, otherwise undefined
               behavior may occur when terminal operation commences.
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `LongStream`

        See
        - .longStream(java.util.Spliterator.OfLong, boolean)
        """
        ...


    @staticmethod
    def doubleStream(spliterator: "Spliterator.OfDouble", parallel: bool) -> "DoubleStream":
        """
        Creates a new sequential or parallel `DoubleStream` from a
        `Spliterator.OfDouble`.
        
        The spliterator is only traversed, split, or queried for estimated size
        after the terminal operation of the stream pipeline commences.
        
        It is strongly recommended the spliterator report a characteristic of
        `IMMUTABLE` or `CONCURRENT`, or be
        <a href="../Spliterator.html#binding">late-binding</a>.  Otherwise,
        .doubleStream(java.util.function.Supplier, int, boolean) should
        be used to reduce the scope of potential interference with the source.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.

        Arguments
        - spliterator: A `Spliterator.OfDouble` describing the stream elements
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `DoubleStream`
        """
        ...


    @staticmethod
    def doubleStream(supplier: "Supplier"["Spliterator.OfDouble"], characteristics: int, parallel: bool) -> "DoubleStream":
        """
        Creates a new sequential or parallel `DoubleStream` from a
        `Supplier` of `Spliterator.OfDouble`.
        
        The Supplier.get() method will be invoked on the supplier no
        more than once, and only after the terminal operation of the stream pipeline
        commences.
        
        For spliterators that report a characteristic of `IMMUTABLE`
        or `CONCURRENT`, or that are
        <a href="../Spliterator.html#binding">late-binding</a>, it is likely
        more efficient to use .doubleStream(java.util.Spliterator.OfDouble, boolean)
        instead.
        The use of a `Supplier` in this form provides a level of
        indirection that reduces the scope of potential interference with the
        source.  Since the supplier is only invoked after the terminal operation
        commences, any modifications to the source up to the start of the
        terminal operation are reflected in the stream result.  See
        <a href="package-summary.html#NonInterference">Non-Interference</a> for
        more details.

        Arguments
        - supplier: A `Supplier` of a `Spliterator.OfDouble`
        - characteristics: Spliterator characteristics of the supplied
               `Spliterator.OfDouble`.  The characteristics must be equal to
               `supplier.get().characteristics()`, otherwise undefined
               behavior may occur when terminal operation commences.
        - parallel: if `True` then the returned stream is a parallel
               stream; if `False` the returned stream is a sequential
               stream.

        Returns
        - a new sequential or parallel `DoubleStream`

        See
        - .doubleStream(java.util.Spliterator.OfDouble, boolean)
        """
        ...



from .SubscriberRegistry import SubscriberRegistry
from .SubscriberExceptionHandler import SubscriberExceptionHandler
from .SubscriberExceptionContext import SubscriberExceptionContext
from .Subscriber import Subscriber
from .Subscribe import Subscribe
from .ParametricNullness import ParametricNullness
from .EventBus import EventBus
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .Dispatcher import Dispatcher
from .DeadEvent import DeadEvent
from .AsyncEventBus import AsyncEventBus
from .AllowConcurrentEvents import AllowConcurrentEvents